@@ -88,31 +88,29 @@
 
       (set! *warn-on-reflection* true)
 
-;; Allow the file to be reloaded in Clojure, undoing some weirdness below
-      (do (ns-unmap *ns* 'String) (ns-unmap *ns* 'Number)
-          (import 'java.lang.String 'java.lang.Number))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Schema protocol
 
 (defprotocol Schema
-  (check [this x]
-    "Check that x satisfies this schema, returning nil for success or a datum that looks
-     like the 'bad' parts of x with ValidationErrors at the leaves describing the error.
+  (walker [this]
+    "Produce a function that takes [data], and either returns a walked version of data
+     (by default, usually just data), or a utils/ErrorContainer containing value that looks
+     like the 'bad' parts of data with ValidationErrors at the leaves describing the failures.
 
-     Examples:
-     user> (s/check s/Keyword :a)
-     nil
+     If this is a composite schema, should let-bind (subschema-walker sub-schema) for each
+     subschema outside the returned fn.  Within the returned fn, should break down data
+     into constituents, call the let-bound subschema walkers on each component, and then
+     reassemble the components into a walked version of the data (or an ErrorContainer
+     describing the validaiton failures).
 
-     user> (s/check s/Keyword 'a)
-     (not (keyword? a)) ;; pretty-printed validation error in clojure
+     Attempting to walk a value that already contains a utils/ErrorContainer produces undefined
+     behavior.
 
-     user> (s/check {:a s/Keyword :b [s/Int]}
-                    {:a :z        :b [1 :whoops 3]})
-     {:b [nil (not (integer? :whoops)) nil]}")
+     User code should never call `walker` directly.  Instead, it should call `start-walker`
+     below.")
   (explain [this]
     "Expand this schema to a human-readable format suitable for pprinting,
-     also expanding classes schematas at the leaves.  Example:
+     also expanding class schematas at the leaves.  Example:
 
      user> (s/explain {:a s/Keyword :b [s/Int]} )
      {:a Keyword, :b [Int]}"))
@@ -125,6 +123,41 @@
     (prefer-method print-method schema.core.Schema java.util.Map)
     (prefer-method print-method schema.core.Schema clojure.lang.IPersistentMap))
 
+(def ^:dynamic subschema-walker
+  "The function to call within 'walker' implementations to create walkers for subschemas.
+   Can be dynamically bound (using start-walker below) to create different walking behaviors.
+
+   For the curious, implemented using dynamic binding rather than making walker take a
+   subschema-walker as an argument because some behaviors (e.g. recursive schema walkers)
+   seem to require mind-bending things like fixed-point combinators that way, but are
+   simple this way."
+  (fn [s]
+    (macros/error!
+     (str "Walking is unsupported outside of start-walker; "
+          "all composite schemas must eagerly bind subschema-walkers "
+          "outside the returned walker."))))
+
+(defn start-walker
+  "The entry point for creating walkers.  Binds the provided walker to subschema-walker,
+   then calls it on the provided schema.  For simple validation, pass walker as sub-walker.
+   More sophisticated behavior (coercion, etc), can be achieved by passing a sub-walker
+   that wraps walker with additional behavior."
+  [sub-walker schema]
+  (binding [subschema-walker sub-walker]
+    (subschema-walker schema)))
+
+(defn checker
+  "Compile an efficient checker for schema, which returns nil for valid values and
+   error descriptions otherwise."
+  [schema]
+  (comp utils/error-val (start-walker walker schema)))
+
+(defn check
+  "Return nil if x matches schema; otherwise, returns a value that looks like the
+   'bad' parts of x with ValidationErrors at the leaves describing the failures."
+  [schema x]
+  ((checker schema) x))
+
 (defn validate
   "Throw an exception if value does not satisfy schema; otherwise, return value."
   [schema value]
@@ -133,66 +166,90 @@
                    {:schema schema :value value :error error}))
   value)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Platform-specific leaf Schemas
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Simple Schemas
+;; On the JVM, a Class itself is a schema. In JS, we treat functions as prototypes so any
+;; function prototype checks objects for compatibility.
 
-;;; The schemas here are defined as records (rather than reify), so they behave like data
-;;; and print reasonably (even when not going through print-method/explain above).
+(extend-protocol Schema
+        Class
+                    
+  (walker [this]
+    (let [class-walker (if-let [more-schema (utils/class-schema this)]
+                         ;; do extra validation for records and such
+                         (subschema-walker more-schema)
+                         identity)]
+      (fn [x]
+        (or (when       (not (instance? this x))
+                                     
+                                                                         
+                                                                          
+                  (macros/validation-error this x (list 'instance? this (utils/value-name x))))
+            (class-walker x)))))
+  (explain [this]
+    (if-let [more-schema (utils/class-schema this)]
+      (explain more-schema)
+            (symbol (.getName ^Class this))            )))
 
 
-;;; Any(thing)
+;; On the JVM, the primitive coercion functions (double, long, etc)
+;; alias to the corresponding boxed number classes
+
+     
+(do
+  (defmacro extend-primitive [cast-sym class-sym]
+    `(extend-protocol Schema
+       ~cast-sym
+       (walker [this#]
+         (subschema-walker ~class-sym))
+       (explain [this#]
+         (explain ~class-sym))))
+
+  (extend-primitive clojure.core$double Double)
+  (extend-primitive clojure.core$float Float)
+  (extend-primitive clojure.core$long Long)
+  (extend-primitive clojure.core$int Integer)
+  (extend-primitive clojure.core$short Short)
+  (extend-primitive clojure.core$char Character)
+  (extend-primitive clojure.core$byte Byte)
+  (extend-primitive clojure.core$boolean Boolean)
+
+  (extend-primitive clojure.core$doubles (Class/forName "[D"))
+  (extend-primitive clojure.core$floats (Class/forName "[F"))
+  (extend-primitive clojure.core$longs (Class/forName "[J"))
+  (extend-primitive clojure.core$ints (Class/forName "[I"))
+  (extend-primitive clojure.core$shorts (Class/forName "[S"))
+  (extend-primitive clojure.core$chars (Class/forName "[C"))
+  (extend-primitive clojure.core$bytes (Class/forName "[B"))
+  (extend-primitive clojure.core$booleans (Class/forName "[Z")))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Cross-platform Schema leaves
+
+;;; Any matches anything (including nil)
 
 (defrecord AnythingSchema [_]
   ;; _ is to work around bug in Clojure where eval-ing defrecord with no fields
   ;; loses type info, which makes this unusable in schema-fn.
   ;; http://dev.clojure.org/jira/browse/CLJ-1196
   Schema
-  (check [this x] nil)
+  (walker [this] identity)
   (explain [this] 'Any))
 
 (def Any
   "Any value, including nil."
   (AnythingSchema. nil))
 
-
-;;; maybe (nil)
-
-(defrecord Maybe [schema]
-  Schema
-  (check [this x]
-    (when-not (nil? x)
-      (check schema x)))
-  (explain [this] (list 'maybe (explain schema))))
-
-(defn maybe
-  "A value that must either be nil or satisfy schema"
-  [schema]
-  (Maybe. schema))
-
-
-;;; named (schema elements)
-
-(defrecord NamedSchema [schema name]
-  Schema
-  (check [this x]
-    (when-let [err (check schema x)]
-      (utils/->NamedError name err)))
-  (explain [this] (list 'named (explain schema) name)))
-
-(defn named
-  "A value that must satisfy schema, and has a name for documentation purposes."
-  [schema name]
-  (NamedSchema. schema name))
-
-
 ;;; eq (to a single allowed value)
 
 (defrecord EqSchema [v]
   Schema
-  (check [this x]
-    (when-not (= v x)
-      (macros/validation-error this x (list '= v (utils/value-name x)))))
+  (walker [this]
+    (fn [x]
+      (if (= v x)
+        x
+        (macros/validation-error this x (list '= v (utils/value-name x))))))
   (explain [this] (list 'eq v)))
 
 (defn eq
@@ -205,9 +262,11 @@
 
 (defrecord EnumSchema [vs]
   Schema
-  (check [this x]
-    (when-not (contains? vs x)
-      (macros/validation-error this x (list vs (utils/value-name x)))))
+  (walker [this]
+    (fn [x]
+      (if (contains? vs x)
+        x
+        (macros/validation-error this x (list vs (utils/value-name x))))))
   (explain [this] (cons 'enum vs)))
 
 (defn enum
@@ -216,70 +275,15 @@
   (EnumSchema. (set vs)))
 
 
-;;; either (satisfies this schema or that one)
-
-(defrecord Either [schemas]
-  Schema
-  (check [this x]
-    (when (every? #(check % x) schemas)
-      (macros/validation-error this x
-                               (list 'every? (list 'check '% (utils/value-name x)) 'schemas))))
-  (explain [this] (cons 'either (map explain schemas))))
-
-(defn either
-  "A value that must satisfy at least one schema in schemas."
-  [& schemas]
-  (Either. schemas))
-
-
-;;; both (satisfies this schema and that one)
-
-(defrecord Both [schemas]
-  Schema
-  (check [this x]
-    (when-let [errors (seq (keep #(check % x) schemas))]
-      (if (= 1 (count errors))
-        (first errors)
-        (macros/validation-error this x (list 'empty? (vec errors))))))
-  (explain [this] (cons 'both (map explain schemas))))
-
-(defn both
-  "A value that must satisfy every schema in schemas."
-  [& schemas]
-  (Both. schemas))
-
-
-;;; protocol (which value must satisfy?)
-
-(defn protocol-name [protocol]
-        (-> protocol :p :var meta :name)
-                                      )
-
-;; In cljs, satisfies? is a macro so we must precompile (partial satisfies? p)
-;; and put it in metadata of the record so that equality is preserved, along with the name.
-(defrecord Protocol [p]
-  Schema
-  (check [this x]
-    (when-not       (satisfies? p x)                                     
-              (macros/validation-error this x (list 'satisfies? (protocol-name this) (utils/value-name x)))))
-  (explain [this] (list 'protocol (protocol-name this))))
-
-      ;; The cljs version is macros/protocol
-(defn protocol
-  "A value that must satsify? protocol p"
-  [p]
-  (macros/assert! (:on p) "Cannot make protocol schema for non-protocol %s" p)
-  (Protocol. p))
-
-
-;;; predicate (which must return truthy for value)
+;;; pred (matches all values for which p? returns truthy)
 
 (defrecord Predicate [p? pred-name]
   Schema
-  (check [this x]
-    (when-let [reason (try (when-not (p? x) 'not)
-                           (catch       Exception                 e 'throws?))]
-      (macros/validation-error this x (list pred-name (utils/value-name x)) reason)))
+  (walker [this]
+    (fn [x]
+      (if-let [reason (macros/try-catchall (when-not (p? x) 'not) (catch e 'throws?))]
+        (macros/validation-error this x (list pred-name (utils/value-name x)) reason)
+        x)))
   (explain [this]
     (cond (= p? integer?) 'Int
           (= p? keyword?) 'Keyword
@@ -295,14 +299,180 @@
      (Predicate. p? pred-name)))
 
 
+;;; protocol (which value must `satisfies?`)
+
+(defn protocol-name [protocol]
+        (-> protocol :p :var meta :name)
+                                      )
+
+;; In cljs, satisfies? is a macro so we must precompile (partial satisfies? p)
+;; and put it in metadata of the record so that equality is preserved, along with the name.
+(defrecord Protocol [p]
+  Schema
+  (walker [this]
+    (fn [x]
+      (if       (satisfies? p x)                                     
+          x
+          (macros/validation-error this x (list 'satisfies? (protocol-name this) (utils/value-name x))))))
+  (explain [this] (list 'protocol (protocol-name this))))
+
+      ;; The cljs version is macros/protocol by necessity, since cljs `satisfies?` is a macro.
+(defn protocol
+  "A value that must satsify? protocol p"
+  [p]
+  (macros/assert! (:on p) "Cannot make protocol schema for non-protocol %s" p)
+  (Protocol. p))
+
+
+;;; regex (validates matching Strings)
+
+(extend-protocol Schema
+        java.util.regex.Pattern
+                  
+  (walker [this]
+    (fn [x]
+      (cond (not (string? x))
+            (macros/validation-error this x (list 'string? (utils/value-name x)))
+
+            (not (re-find this x))
+            (macros/validation-error this x (list 're-find
+                                                  (explain this)
+                                                  (utils/value-name x)))
+
+            :else x)))
+  (explain [this]
+          (symbol (str "#\"" this "\""))
+                                                             ))
+
+
+;;; Cross-platform Schemas for atomic value types
+
+(def Str
+  "Satisfied only by String.
+   Is (pred string?) and not js/String in cljs because of keywords."
+        java.lang.String                      )
+
+(def Num
+  "Any number"
+        java.lang.Number                 )
+
+(def Int
+  "Any integral number"
+  (pred integer? 'integer?))
+
+(def Keyword
+  "A keyword"
+  (pred keyword? 'keyword?))
+
+(def Regex
+  "A regular expression"
+        java.util.regex.Pattern                 )
+
+(def Inst
+  "The local representation of #inst ..."
+        java.util.Date               )
+
+(def Uuid
+  "The local representation of #uuid ..."
+        java.util.UUID                      )
+
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Simple composite Schemas
+
+;;; maybe (nil)
+
+(defrecord Maybe [schema]
+  Schema
+  (walker [this]
+    (let [sub-walker (subschema-walker schema)]
+      (fn [x]
+        (when-not (nil? x)
+          (sub-walker x)))))
+  (explain [this] (list 'maybe (explain schema))))
+
+(defn maybe
+  "A value that must either be nil or satisfy schema"
+  [schema]
+  (Maybe. schema))
+
+
+;;; named (schema elements)
+
+(defrecord NamedSchema [schema name]
+  Schema
+  (walker [this]
+    (let [sub-walker (subschema-walker schema)]
+      (fn [x] (utils/wrap-error-name name (sub-walker x)))))
+  (explain [this] (list 'named (explain schema) name)))
+
+(defn named
+  "A value that must satisfy schema, and has a name for documentation purposes."
+  [schema name]
+  (NamedSchema. schema name))
+
+
+;;; either (satisfies this schema or that one)
+
+(defrecord Either [schemas]
+  Schema
+  (walker [this]
+    (let [sub-walkers (mapv subschema-walker schemas)]
+      (fn [x]
+        (loop [sub-walkers (seq sub-walkers)]
+          (if-not sub-walkers
+            (macros/validation-error
+             this x
+             (list 'every? (list 'check '% (utils/value-name x)) 'schemas))
+            (let [res ((first sub-walkers) x)]
+              (if-not (utils/error? res)
+                res
+                (recur (next sub-walkers)))))))))
+  (explain [this] (cons 'either (map explain schemas))))
+
+(defn either
+  "A value that must satisfy at least one schema in schemas."
+  [& schemas]
+  (Either. schemas))
+
+
+;;; both (satisfies this schema and that one)
+
+(defrecord Both [schemas]
+  Schema
+  (walker [this]
+    (let [sub-walkers (mapv subschema-walker schemas)]
+      ;; Both doesn't really have a clean semantics for non-identity walks, but we can
+      ;; do something pretty reasonable and assume we walk in order passing the result
+      ;; of each walk to the next, and failing at the first error
+      (fn [x]
+        (reduce
+         (fn [x sub-walker]
+           (if (utils/error? x)
+             x
+             (sub-walker x)))
+         x
+         sub-walkers))))
+  (explain [this] (cons 'both (map explain schemas))))
+
+(defn both
+  "A value that must satisfy every schema in schemas."
+  [& schemas]
+  (Both. schemas))
+
+
 ;;; conditional (choice of schema, based on predicates on the value)
 
 (defrecord ConditionalSchema [preds-and-schemas]
   Schema
-  (check [this x]
-    (if-let [[_ match] (first (filter (fn [[pred]] (pred x)) preds-and-schemas))]
-      (check match x)
-      (macros/validation-error this x (list 'matches-some-condition? (utils/value-name x)))))
+  (walker [this]
+    (let [preds-and-walkers (mapv (fn [[pred schema]] [pred (subschema-walker schema)])
+                                  preds-and-schemas)]
+      (fn [x]
+        (if-let [[_ match] (first (filter (fn [[pred]] (pred x)) preds-and-walkers))]
+          (match x)
+          (macros/validation-error this x (list 'matches-some-condition? (utils/value-name x)))))))
   (explain [this]
     (->> preds-and-schemas
          (mapcat (fn [[pred schema]] [pred (explain schema)]))
@@ -332,24 +502,29 @@
 ;; Clojure (but not ClojureScript) vars.
 
      
-(do (defn var-name [v]
-      (let [{:keys [ns name]} (meta v)]
-        (symbol (str (ns-name ns) "/" name))))
+(do
+  (defn var-name [v]
+    (let [{:keys [ns name]} (meta v)]
+      (symbol (str (ns-name ns) "/" name))))
 
-    (defrecord Recursive [schema-var]
-      Schema
-      (check [this x]
-        (check @schema-var x))
-      (explain [this]
-        (list 'recursive (list 'var (var-name schema-var)))))
+  (defrecord Recursive [schema-var]
+    Schema
+    (walker [this]
+      (let [a (atom nil)]
+        (reset! a (start-walker
+                   (let [old subschema-walker]
+                     (fn [s] (if (= s this) #(@a %) (old s))))
+                   @schema-var))))
+    (explain [this]
+      (list 'recursive (list 'var (var-name schema-var)))))
 
-    (defn recursive
-      "Support for (mutually) recursive schemas by passing a var that points to a schema,
+  (defn recursive
+    "Support for (mutually) recursive schemas by passing a var that points to a schema,
        e.g (recursive #'ExampleRecursiveSchema)."
-      [schema-var]
-      (when-not (var? schema-var)
-        (macros/error! (utils/format* "Not a var: %s" schema-var)))
-      (Recursive. schema-var)))
+    [schema-var]
+    (when-not (var? schema-var)
+      (macros/error! (utils/format* "Not a var: %s" schema-var)))
+    (Recursive. schema-var)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Map Schemas
@@ -372,7 +547,11 @@
 ;; mappings from integers to strings.
 
 
-;;; Definitions for required and optional keys
+;;; Definitions for required and optional keys, and single entry validators
+
+(def +missing+
+  "A sentinel value representing missing portions of the input data."
+  ::missing)
 
 (defrecord RequiredKey [k])
 
@@ -397,7 +576,6 @@
 (defn optional-key? [ks]
   (instance? OptionalKey ks))
 
-;;; Implementation helper functions
 
 (defn explicit-schema-key [ks]
   (cond (keyword? ks) ks
@@ -409,6 +587,62 @@
   (or (required-key? ks)
       (optional-key? ks)))
 
+(defn- explain-kspec [kspec]
+  (if (specific-key? kspec)
+    (if (keyword? kspec)
+      kspec
+      (list (cond (required-key? kspec) 'required-key
+                  (optional-key? kspec) 'optional-key)
+            (explicit-schema-key kspec)))
+    (explain kspec)))
+
+;; A schema for a single map entry.  kspec is either a keyword, required or optional key,
+;; or key schema.  val-schema is a value schema.
+(defrecord MapEntry [kspec val-schema]
+  Schema
+  (walker [this]
+    (let [val-walker (subschema-walker val-schema)]
+      (if (specific-key? kspec)
+        (let [optional? (optional-key? kspec)
+              k (explicit-schema-key kspec)]
+          (fn [x]
+            (cond (identical? +missing+ x)
+                  (when-not optional?
+                    (utils/error [k 'missing-required-key]))
+
+                  (not (= 2 (count x)))
+                  (macros/validation-error this x (list = 2 (list 'count (utils/value-name x))))
+
+                  :else
+                  (let [[xk xv] x]
+                    (assert (= xk k))
+                    (let [vres (val-walker xv)]
+                      (if-let [ve (utils/error-val vres)]
+                        (utils/error [xk ve])
+                        [xk vres]))))))
+        (let [key-walker (subschema-walker kspec)]
+          (fn [x]
+            (if-not (= 2 (count x))
+              (macros/validation-error this x (list = 2 (list 'count (utils/value-name x))))
+              (let [out-k (key-walker (key x))
+                    out-ke (utils/error-val out-k)
+                    out-v (val-walker (val x))
+                    out-ve (utils/error-val out-v)]
+                (if (or out-ke out-ve)
+                  (utils/error [(or out-ke (key x)) (or out-ve 'invalid-key)])
+                  [out-k out-v]))))))))
+  (explain [this]
+    (list
+     'map-entry
+     (explain-kspec kspec)
+     (explain val-schema))))
+
+(defn map-entry [kspec val-schema]
+  (MapEntry. kspec val-schema))
+
+
+;;; Implementation helper functions
+
 (defn- find-extra-keys-schema [map-schema]
   (let [key-schemata (remove specific-key? (keys map-schema))]
     (macros/assert! (< (count key-schemata) 2)
@@ -416,68 +650,51 @@
                     (vec key-schemata))
     (first key-schemata)))
 
-(defn- check-explicit-key
-  "Validate a single schema key and dissoc the value from m"
-  [value [key-schema val-schema]]
-  (let [optional? (optional-key? key-schema)
-        k (explicit-schema-key key-schema)
-        present? (contains? value k)]
-    (cond (and (not optional?) (not present?))
-          [k 'missing-required-key]
-
-          present?
-          (when-let [error (check val-schema (get value k))]
-            [k error]))))
-
-(defn- check-extra-key
-  "Validate a single schema key and dissoc the value from m"
-  [key-schema val-schema [value-k value-v]]
-  (if-not key-schema
-    [value-k 'disallowed-key]
-    (if-let [error (check key-schema value-k)]
-      [error 'invalid-key]
-      (when-let [error (check val-schema value-v)]
-        [value-k error]))))
-
-(defn- check-map [map-schema value]
+(defn- map-walker [map-schema]
   (let [extra-keys-schema (find-extra-keys-schema map-schema)
-        extra-vals-schema (get map-schema extra-keys-schema)
+        extra-walker (when extra-keys-schema
+                       (subschema-walker (apply map-entry (find map-schema extra-keys-schema))))
         explicit-schema (dissoc map-schema extra-keys-schema)
-        errors (concat
-                (keep #(check-explicit-key value %)
-                      explicit-schema)
-                (keep #(check-extra-key extra-keys-schema extra-vals-schema %)
-                      (apply dissoc value (map explicit-schema-key (keys explicit-schema)))))]
-    (when (seq errors)
-      (into {} errors))))
-
-
-;;; Extending the Schema protocol to Clojure maps.
-
-(defn- map-check [this x]
-  (if-not (map? x)
-    (macros/validation-error this x (list 'map? (utils/value-name x)))
-    (check-map this x)))
+        explicit-walkers (into {} (for [[k v] explicit-schema]
+                                    [(explicit-schema-key k)
+                                     (subschema-walker (map-entry k v))]))
+        err-conj (utils/result-builder (constantly {}))]
+    (when-not (= (count explicit-schema) (count explicit-walkers))
+      (macros/error! (utils/format* "Schema has multiple variants of the same explicit key: %s"
+                                    (->> (keys explicit-schema)
+                                         (group-by explicit-schema-key)
+                                         vals
+                                         (filter #(> (count %) 1))
+                                         (apply concat)
+                                         (mapv explain-kspec)))))
+    (fn [x]
+      (if-not (map? x)
+        (macros/validation-error map-schema x (list 'map? (utils/value-name x)))
+        (loop [x x explicit-walkers (seq explicit-walkers) out {}]
+          (if-not explicit-walkers
+            (reduce
+             (if extra-walker
+               (fn [out e]
+                 (err-conj out (extra-walker e)))
+               (fn [out [k _]]
+                 (err-conj out (utils/error [k 'disallowed-key]))))
+             out
+             x)
+            (let [[wk wv] (first explicit-walkers)]
+              (recur (dissoc x wk)
+                     (next explicit-walkers)
+                     (err-conj out (wv (or (find x wk) +missing+)))))))))))
 
 (defn- map-explain [this]
-  (into {}
-        (for [[k v] this]
-          [(if (specific-key? k)
-             (if (keyword? k)
-               k
-               (list (cond (required-key? k) 'required-key
-                           (optional-key? k) 'optional-key)
-                     (explicit-schema-key k)))
-             (explain k))
-           (explain v)])))
+  (into {} (for [[k v] this] (vec (next (explain (map-entry k v)))))))
 
 (extend-protocol Schema
         clojure.lang.APersistentMap
                                      
-  (check [this x] (map-check this x))
+  (walker [this] (map-walker this))
   (explain [this] (map-explain this))
                                     
-                                            
+                                          
                                             )
 
 
@@ -489,12 +706,16 @@
 (extend-protocol Schema
         clojure.lang.APersistentSet
                                     
-  (check [this x]
+  (walker [this]
     (macros/assert! (= (count this) 1) "Set schema must have exactly one element")
-    (or (when-not (set? x)
-          (macros/validation-error this x (list 'set? (utils/value-name x))))
-        (when-let [out (seq (keep #(check (first this) %) x))]
-          (set out))))
+    (let [sub-walker (subschema-walker (first this))]
+      (fn [x]
+        (or (when-not (set? x)
+              (macros/validation-error this x (list 'set? (utils/value-name x))))
+            (let [[good bad] ((juxt remove keep) utils/error-val (map sub-walker x))]
+              (if (seq bad)
+                (utils/error (set bad))
+                (set good)))))))
   (explain [this] (set [(explain (first this))])))
 
 
@@ -530,37 +751,41 @@
 (extend-protocol Schema
         clojure.lang.APersistentVector
                                    
-  (check [this x]
-    (or (when-not (or (nil? x) (sequential? x))
-          (macros/validation-error this x (list 'sequential? (utils/value-name x))))
-        (let [[singles multi] (parse-sequence-schema this)]
-          (#(when (some identity %) %)
-           (loop [singles singles x x out []]
-             (if-let [[^One first-single & more-singles] (seq singles)]
-               (if (empty? x)
-                 (if (.-optional? first-single)
-                   out
-                   (conj out
-                         (macros/validation-error
-                          (vec singles)
-                          nil
-                          (list* 'present?
-                                 (for [^One single singles
-                                       :while (not (.-optional? single))]
-                                   (.-name single))))))
-                 (recur more-singles
-                        (rest x)
-                        (conj out
-                              (when-let [err (check (.-schema first-single) (first x))]
-                                (utils/->NamedError (.-name first-single) err)))))
-               (cond multi
-                     (into out (map #(check multi %) x))
+  (walker [this]
+    (let [[singles multi] (parse-sequence-schema this)
+          single-walkers (vec (for [^One s singles]
+                                [s (subschema-walker (.-schema s))]))
+          multi-walker (when multi (subschema-walker multi))
+          err-conj (utils/result-builder (fn [m] (vec (repeat (count m) nil))))]
+      (fn [x]
+        (or (when-not (or (nil? x) (sequential? x))
+              (macros/validation-error this x (list 'sequential? (utils/value-name x))))
+            (loop [single-walkers single-walkers x x out []]
+              (if-let [[^One first-single single-walker] (first single-walkers)]
+                (if (empty? x)
+                  (if (.-optional? first-single)
+                    out
+                    (err-conj out
+                              (macros/validation-error
+                               (vec (map first single-walkers))
+                               nil
+                               (list* 'present?
+                                      (for [[^One single] single-walkers
+                                            :while (not (.-optional? single))]
+                                        (.-name single))))))
+                  (recur (next single-walkers)
+                         (rest x)
+                         (err-conj out
+                                   (utils/wrap-error-name
+                                    (.-name first-single)
+                                    (single-walker (first x))))))
+                (cond multi
+                      (reduce err-conj out (map multi-walker x))
 
-                     (seq x)
-                     (conj out (macros/validation-error nil x (list 'has-extra-elts? (count x))))
-
-                     :else
-                     out)))))))
+                      (seq x)
+                      (err-conj out (macros/validation-error nil x (list 'has-extra-elts? (count x))))
+                      :else
+                      out)))))))
   (explain [this]
     (let [[singles multi] (parse-sequence-schema this)]
       (vec
@@ -586,12 +811,20 @@
 
 (defrecord Record [klass schema]
   Schema
-  (check [this r]
-    (or (when-not (instance? klass r)
-          (macros/validation-error this r (list 'instance? klass (utils/value-name r))))
-        (check-map schema r)
-        (when-let [f (:extra-validator-fn this)]
-          (check (pred f) r))))
+  (walker [this]
+    (let [map-checker (subschema-walker schema)
+          pred-checker (when-let [evf (:extra-validator-fn this)]
+                         (subschema-walker (pred evf)))]
+      (fn [r]
+        (or (when-not (instance? klass r)
+              (macros/validation-error this r (list 'instance? klass (utils/value-name r))))
+            (let [res (map-checker r)]
+              (if (utils/error? res)
+                res
+                (let [pred-res (when pred-checker (pred-checker r))]
+                  (if (utils/error? pred-res)
+                    pred-res
+                    (merge r res)))))))))
   (explain [this]
     (list 'record       (symbol (.getName ^Class klass))                                (explain schema))))
 
@@ -620,9 +853,11 @@
 
 (defrecord FnSchema [output-schema input-schemas] ;; input-schemas sorted by arity
   Schema
-  (check [this x] nil
-    (when-not (fn? x)
-      (macros/validation-error this x (list 'fn? (utils/value-name x)))))
+  (walker [this]
+    (fn [x]
+      (if (fn? x)
+        x
+        (macros/validation-error this x (list 'fn? (utils/value-name x))))))
   (explain [this]
     (if (> (count input-schemas) 1)
       (list* '=>* (explain output-schema) (map explain-input-schema input-schemas))
@@ -653,129 +888,18 @@
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Cross-platform Schema leaves, for writing Schemas that are valid in both clj and cljs.
+;;; Schematized defrecord and (de,let)fn macros
 
-      (ns-unmap *ns* 'String)
-(def String
-  "Satisfied only by String.
-   Is (pred string?) and not js/String in cljs because of keywords."
-        java.lang.String                      )
-
-      (ns-unmap *ns* 'Number)
-(def Number
-  "Any number"
-        java.lang.Number                 )
-
-(def Int
-  "Any integral number"
-  (pred integer? 'integer?))
-
-(def Keyword
-  "A keyword"
-  (pred keyword? 'keyword?))
-
-(def Regex
-  "Any Regular Expression"
-        java.util.regex.Pattern                 )
-
-(def Inst
-  "The local representation of #inst ..."
-        java.util.Date               )
-
-(def Uuid
-  "The local representation of #uuid ..."
-        java.util.UUID                      )
-
-
-;;; Extending the Schema protocol to java.util.regex.Pattern
-
-(defn- explain-regex [regex]
-        (symbol (str "#\"" regex "\""))
-                                                             )
-
-(extend-protocol Schema
-        java.util.regex.Pattern
-                  
-  (check [this x]
-    (if-not (string? x)
-      (macros/validation-error this x (list 'string? (utils/value-name x)))
-      (when-not (re-find this x)
-        (macros/validation-error this x (list 're-find
-                                              (explain-regex this)
-                                              (utils/value-name x))))))
-  (explain [this]
-    (explain-regex this)))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Platform-specific Schemas
-
-;; On the JVM, a Class itself is a schema, and the primitive coercion functions (double, long,
-;; etc) match only instances of that concrete type (identical to Double, Long, etc).
-     
-(do
-  (defn- check-class [schema class value]
-    (when-not (instance? class value)
-      (macros/validation-error schema value (list 'instance? class (utils/value-name value)))))
-
-  (extend-protocol Schema
-    Class
-    (check [this x]
-      (or (check-class this this x)
-          (when-let [more-schema (utils/class-schema this)]
-            (check more-schema x))))
-    (explain [this]
-      (if-let [more-schema (utils/class-schema this)]
-        (explain more-schema)
-        (symbol (.getName ^Class this)))))
-
-  ;; prevent coersion, so you have to be exactly the given type.
-  (defmacro extend-primitive [cast-sym class-sym]
-    `(extend-protocol Schema
-       ~cast-sym
-       (check [this# x#]
-         (check-class ~cast-sym ~class-sym x#))
-       (explain [this#] '~(symbol (last (.split (name cast-sym) "\\$"))))))
-
-  (extend-primitive clojure.core$double Double)
-  (extend-primitive clojure.core$float Float)
-  (extend-primitive clojure.core$long Long)
-  (extend-primitive clojure.core$int Integer)
-  (extend-primitive clojure.core$short Short)
-  (extend-primitive clojure.core$char Character)
-  (extend-primitive clojure.core$byte Byte)
-  (extend-primitive clojure.core$boolean Boolean))
-
-;; On JS, we treat functions as prototypes so any function prototype checks
-;; objects for compatibility (Prototype constructor hack)
-      
-                       
-             
-                 
-                                              
-                      
-                             
-                        
-                                                            
-                                                             
-                                                                                       
-                 
-                                                   
-                           
-             
-
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Schematized records and functions
-
-;; In Clojure, we can suck the defrecord/fn/defn macros into this namespace
+;; In Clojure, we can suck the defrecord/fn/defn/let macros into this namespace
 ;; In ClojureScript, you have to use them from clj schema.macros
      
 (do
-  (doseq [s ['fn 'defn 'defrecord]] (ns-unmap *ns* s))
+  (doseq [s ['fn 'defn 'letfn 'defrecord]] (ns-unmap *ns* s))
   (potemkin/import-vars
    macros/defrecord
    macros/fn
    macros/defn
+   macros/letfn
    macros/with-fn-validation)
   (reset! macros/*use-potemkin* true) ;; Use potemkin for s/defrecord by default.
   (set! *warn-on-reflection* false))
@@ -811,4 +935,4 @@
 
       (potemkin/import-vars macros/defschema)
 
-;;;;;;;;;;;; This file autogenerated from src/cljx/schema/core.cljx
+;;;;;;;;;;;; This file autogenerated from /Users/w01fe/prismatic/open-source/schema/src/cljx/schema/core.cljx
