@@ -3,19 +3,23 @@ require 'puppet/network/http_pool'
 require 'puppet/util/puppetdb'
 require 'puppet/util/puppetdb/command_names'
 require 'puppet/util/puppetdb/char_encoding'
+require 'json'
 
 class Puppet::Util::Puppetdb::Command
+  include Puppet::Util::Puppetdb
   include Puppet::Util::Puppetdb::CommandNames
 
   Url                = "/v3/commands"
 
   # Public instance methods
 
-  # Constructor;
+  # Initialize a Command object, for later submission.
   #
   # @param command String the name of the command; should be one of the
   #   constants defined in `Puppet::Util::Puppetdb::CommandNames`
   # @param version Integer the command version number
+  # @param certname The certname that this command operates on (is not
+  #   included in the actual submission)
   # @param payload Object the payload of the command.  This object should be a
   #   primitive (numeric type, string, array, or hash) that is natively supported
   #   by JSON serialization / deserialization libraries.
@@ -23,24 +27,31 @@ class Puppet::Util::Puppetdb::Command
     @command = command
     @version = version
     @certname = certname
-    @payload = self.class.format_payload(command, version, payload)
+    profile "Format payload" do
+      @payload = self.class.format_payload(command, version, payload)
+    end
   end
 
   attr_reader :command, :version, :certname, :payload
 
+  # Submit the command, returning the result hash.
+  #
+  # @return [Hash <String, String>]
   def submit
     checksum = Digest::SHA1.hexdigest(payload)
-    escaped_payload = CGI.escape(payload)
+
     for_whom = " for #{certname}" if certname
 
     begin
-      http = Puppet::Network::HttpPool.http_instance(config.server, config.port)
-      response = http.post(Url, "checksum=#{checksum}&payload=#{escaped_payload}", headers)
+      response = profile "Submit command HTTP post" do
+        http = Puppet::Network::HttpPool.http_instance(config.server, config.port)
+        http.post(Url + "?checksum=#{checksum}", payload, headers)
+      end
 
       Puppet::Util::Puppetdb.log_x_deprecation_header(response)
 
       if response.is_a? Net::HTTPSuccess
-        result = PSON.parse(response.body)
+        result = JSON.parse(response.body)
         Puppet.info "'#{command}' command#{for_whom} submitted to PuppetDB with UUID #{result['uuid']}"
         result
       else
@@ -88,7 +99,7 @@ class Puppet::Util::Puppetdb::Command
   def headers
     {
       "Accept" => "application/json",
-      "Content-Type" => "application/x-www-form-urlencoded; charset=UTF-8",
+      "Content-Type" => "application/json",
     }
   end
 
