@@ -3,6 +3,7 @@ require 'spec_helper'
 
 require 'puppet/indirector/facts/puppetdb'
 require 'puppet/util/puppetdb/command_names'
+require 'json'
 
 describe Puppet::Node::Facts::Puppetdb do
 
@@ -18,6 +19,7 @@ describe Puppet::Node::Facts::Puppetdb do
     let(:response) { Net::HTTPOK.new('1.1', 200, 'OK') }
     let(:facts)    { Puppet::Node::Facts.new('foo') }
     let(:http)     { mock 'http' }
+    let(:env)      { "my_environment" }
 
     before :each do
       Puppet::Network::HttpPool.expects(:http_instance).returns http
@@ -25,40 +27,42 @@ describe Puppet::Node::Facts::Puppetdb do
     end
 
     def save
-      subject.save(Puppet::Node::Facts.indirection.request(:save, facts.name, facts))
+      subject.save(Puppet::Node::Facts.indirection.request(:save, facts.name, facts, :environment => env))
     end
 
-    it "should POST the facts command as a URL-encoded PSON string" do
+    it "should POST the facts as a JSON string" do
       facts.stringify
-      f = {"name" => facts.name, "values" => facts.values}
+      f = {
+        "name" => facts.name,
+        "values" => facts.values,
+        "environment" => env,
+      }
+
       payload = {
         :command => CommandReplaceFacts,
-        :version => 1,
-        :payload => f.to_pson,
-      }.to_pson
+        :version => 2,
+        :payload => f,
+      }.to_json
 
       http.expects(:post).with do |uri, body, headers|
-        body =~ /payload=(.+)/
-        @sent_payload = $1
+        expect(body).to eq(payload)
       end.returns response
 
       save
-
-      CGI.unescape(@sent_payload).should == payload
     end
 
     it "should stringify fact values before submitting" do
       facts.values['something'] = 100
 
+      sent_payload = nil
       http.expects(:post).with do |uri, body, headers|
-        body =~ /payload=(.+)/
-        @sent_payload = $1
+        sent_payload = body
       end.returns response
 
       save
 
-      message = PSON.parse(CGI.unescape(@sent_payload))
-      sent_facts = PSON.parse(message['payload'])
+      message = JSON.parse(sent_payload)
+      sent_facts = message['payload']
 
       # We shouldn't modify the original instance
       facts.values['something'].should == 100
@@ -73,7 +77,7 @@ describe Puppet::Node::Facts::Puppetdb do
 
     it "should return the facts if they're found" do
       body = [{:node => 'some_node', :name => 'a', :value => '1'},
-              {:node => 'some_node', :name => 'b', :value => '2'}].to_pson
+              {:node => 'some_node', :name => 'b', :value => '2'}].to_json
 
       response = Net::HTTPOK.new('1.1', 200, 'OK')
       response.stubs(:body).returns body
@@ -87,7 +91,7 @@ describe Puppet::Node::Facts::Puppetdb do
     end
 
     it "should return nil if no facts are found" do
-      body = [].to_pson
+      body = [].to_json
 
       response = Net::HTTPOK.new('1.1', 200, 'OK')
       response.stubs(:body).returns body
@@ -120,7 +124,7 @@ describe Puppet::Node::Facts::Puppetdb do
       response = Net::HTTPOK.new('1.1', 200, 'OK')
       response['x-deprecation'] = "This is deprecated!"
 
-      body = [].to_pson
+      body = [].to_json
 
       response.stubs(:body).returns body
 
@@ -172,7 +176,7 @@ describe Puppet::Node::Facts::Puppetdb do
       }
 
       query = CGI.escape(["and", ["=", ["fact", "kernel"], "Linux"],
-                                 ["=", ["fact", "uptime"], "10 days"]].to_pson)
+                                 ["=", ["fact", "uptime"], "10 days"]].to_json)
 
       response.stubs(:body).returns '[]'
 
@@ -188,7 +192,7 @@ describe Puppet::Node::Facts::Puppetdb do
         'facts.kernel.ne' => 'Linux',
       }
 
-      query = CGI.escape(["and", ["not", ["=", ["fact", "kernel"], "Linux"]]].to_pson)
+      query = CGI.escape(["and", ["not", ["=", ["fact", "kernel"], "Linux"]]].to_json)
 
       response.stubs(:body).returns '[]'
 
@@ -204,7 +208,7 @@ describe Puppet::Node::Facts::Puppetdb do
         'facts.kernel' => 'Linux',
       }
 
-      query = CGI.escape(["and", ["=", ["fact", "kernel"], "Linux"]].to_pson)
+      query = CGI.escape(["and", ["=", ["fact", "kernel"], "Linux"]].to_json)
 
       response.stubs(:body).returns '[]'
 
@@ -226,7 +230,7 @@ describe Puppet::Node::Facts::Puppetdb do
           "facts.kernel.#{name}" => 'Linux',
         }
 
-        query = CGI.escape(["and", [operator, ["fact", "kernel"], "Linux"]].to_pson)
+        query = CGI.escape(["and", [operator, ["fact", "kernel"], "Linux"]].to_json)
 
         response.stubs(:body).returns '[]'
 
