@@ -15,6 +15,10 @@
            (s/trim out)
            "0.0-dev-build"))))))
 
+(def tk-version "0.3.4")
+(def tk-jetty9-version "0.3.3")
+(def ks-version "0.5.3")
+
 (defproject puppetdb (version-string)
   :description "Puppet-integrated catalog and fact storage"
 
@@ -23,14 +27,13 @@
   ;; requires lein 2.2.0+.
   :pedantic? :abort
 
-  :dependencies [[org.clojure/clojure "1.5.1"]
+  :dependencies [[org.clojure/clojure "1.6.0"]
                  [cheshire "5.2.0"]
                  [org.clojure/core.match "0.2.0-rc5"]
                  [org.clojure/math.combinatorics "0.0.4"]
                  [org.clojure/tools.logging "0.2.6"]
                  [org.clojure/tools.nrepl "0.2.3"]
                  [puppetlabs/tools.namespace "0.2.4.1"]
-                 [swank-clojure "1.4.3"]
                  [vimclojure/server "2.3.6" :exclusions [org.clojure/clojure]]
                  [clj-stacktrace "0.2.6"]
                  [metrics-clojure "0.7.0" :exclusions [org.clojure/clojure org.slf4j/slf4j-api]]
@@ -44,13 +47,9 @@
                  [overtone/at-at "1.2.0"]
                  ;; Nicer exception handling with try+/throw+
                  [slingshot "0.10.3"]
-                 [log4j "1.2.17" :exclusions [javax.mail/mail
-                                              javax.jms/jms
-                                              com.sun.jdmk/jmxtools
-                                              com.sun.jmx/jmxri]]
+
                  ;; Database connectivity
                  [com.jolbox/bonecp "0.7.1.RELEASE" :exclusions [org.slf4j/slf4j-api]]
-                 [org.slf4j/slf4j-log4j12 "1.7.5"]
                  [org.clojure/java.jdbc "0.1.1"]
                  [org.hsqldb/hsqldb "2.2.8"]
                  [org.postgresql/postgresql "9.2-1003-jdbc4"]
@@ -58,16 +57,19 @@
                  ;; MQ connectivity
                  [clamq/clamq-activemq "0.4" :exclusions [org.slf4j/slf4j-api]]
                  [org.apache.activemq/activemq-core "5.6.0" :exclusions [org.slf4j/slf4j-api org.fusesource.fuse-extra/fusemq-leveldb]]
+                 ;; bridge to allow some spring/activemq stuff to log over slf4j
+                 [org.slf4j/jcl-over-slf4j "1.7.5"]
                  ;; WebAPI support libraries.
                  [net.cgrand/moustache "1.1.0" :exclusions [ring/ring-core org.clojure/clojure]]
                  [clj-http "0.5.3"]
-                 [ring/ring-core "1.1.8"]
-                 [ring/ring-jetty-adapter "1.1.8"]
-                 [org.apache.commons/commons-compress "1.4.1"]
-                 [puppetlabs/kitchensink "0.1.1"]
-                 [org.bouncycastle/bcpkix-jdk15on "1.49"]
+                 [ring/ring-core "1.2.1" :exclusions [javax.servlet/servlet-api]]
+                 [org.apache.commons/commons-compress "1.8"]
+                 [puppetlabs/kitchensink ~ks-version]
+                 [puppetlabs/trapperkeeper ~tk-version]
+                 [puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty9-version]
                  [prismatic/schema "0.2.0"]
-                 [org.clojure/tools.macro "0.1.5"]]
+                 [org.clojure/tools.macro "0.1.5"]
+                 [com.novemberain/pantomime "2.1.0"]]
 
   ;;The below test-selectors is basically using the PUPPETDB_DBTYPE
   ;;environment variable to be the test selector.  The selector below
@@ -82,9 +84,11 @@
 
   :profiles {:dev {:resource-paths ["test-resources"],
                    :dependencies [[ring-mock "0.1.5"]
+                                  [puppetlabs/trapperkeeper ~tk-version :classifier "test"]
+                                  [puppetlabs/kitchensink ~ks-version :classifier "test"]
+                                  [puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty9-version :classifier "test"]
                                   [org.flatland/ordered "1.5.2"]]}}
 
   :jar-exclusions [#"leiningen/"]
 
-  :aot [com.puppetlabs.puppetdb.core clj-time.core]
-  :main com.puppetlabs.puppetdb.core)
+  :main ^:skip-aot com.puppetlabs.puppetdb.core)
