@@ -230,7 +230,7 @@
 
 (defmacro with-transacted-connection'
   "Like `clojure.java.jdbc/with-connection`, except this automatically
-  wraps `body` in a database transaction with the specified transaction 
+  wraps `body` in a database transaction with the specified transaction
   isolation level.  See isolation-levels for possible values."
   [db-spec tx-isolation-level & body]
   `(with-transacted-connection-fn ~db-spec ~tx-isolation-level
@@ -284,7 +284,7 @@
   "Create a new database connection pool"
   [{:keys [classname subprotocol subname user username password
            partition-conn-min partition-conn-max partition-count
-           stats log-statements log-slow-statements
+           stats log-statements log-slow-statements statements-cache-size
            conn-max-age conn-lifetime conn-keep-alive read-only?]
     :as   db}]
   ;; Load the database driver class
@@ -303,6 +303,7 @@
                           ;; paste the URL back together from parts.
                           (.setJdbcUrl (str "jdbc:" subprotocol ":" subname))
                           (.setConnectionHook (connection-hook log-statements log-slow-statements-duration))
+                          (.setStatementsCacheSize statements-cache-size)
                           (.setDefaultReadOnly read-only?))
         user (or user username)]
     ;; configurable without default
