@@ -69,7 +69,9 @@
             [clamq.protocol.consumer :as mq-cons]
             [clamq.protocol.producer :as mq-producer]
             [clamq.protocol.connection :as mq-conn]
-            [com.puppetlabs.jdbc :as jdbc])
+            [com.puppetlabs.jdbc :as jdbc]
+            [clojure.walk :as walk]
+            [com.puppetlabs.puppetdb.utils :as utils])
   (:use [slingshot.slingshot :only [try+ throw+]]
         [cheshire.custom :only (JSONable)]
         [clj-http.util :only [url-encode]]
@@ -159,28 +161,26 @@
 
 (defmulti parse-command
   "Take a wire-format command and parse it into a command object."
-  class)
+  (comp class :body))
 
 (defmethod parse-command (class (byte-array 0))
-  [command-bytes]
-  (parse-command (String. command-bytes "UTF-8")))
+  [bytes-message]
+  (parse-command (update-in bytes-message [:body] #(String. % "UTF-8"))))
 
 (defmethod parse-command String
-  [command-string]
-  {:pre  [(string? command-string)]
+  [{:keys [headers body]}]
+  {:pre  [(string? body)]
    :post [(map? %)
           (:payload %)
           (string? (:command %))
           (number? (:version %))
           (map? (:annotations %))]}
-  (let [message     (json/parse-string command-string true)
-        annotations (get message :annotations {})
-        received    (get annotations :received (kitchensink/timestamp))
-        id          (get annotations :id (kitchensink/uuid))
-        annotations (-> annotations
-                        (assoc :received received)
-                        (assoc :id id))]
-    (assoc message :annotations annotations)))
+  (let [message     (json/parse-string body true)
+        received    (get headers :received (kitchensink/timestamp))
+        id          (get headers :id (kitchensink/uuid))]
+    (-> message
+        (assoc-in [:annotations :received] received)
+        (assoc-in [:annotations :id] id))))
 
 (defn assemble-command
   "Builds a command-map from the supplied parameters"
@@ -222,13 +222,11 @@
             (integer? port)
             (map? command-map)]}
      (let [message (json/generate-string command-map)
-           body    (format "checksum=%s&payload=%s"
-                           (kitchensink/utf8-string->sha1 message)
-                           (url-encode message))
-           url     (format "http://%s:%s/v3/commands" host port)]
-       (client/post url {:body               body
+           checksum (kitchensink/utf8-string->sha1 message)
+           url     (format "http://%s:%s/v4/commands?checksum=%s" host port checksum)]
+       (client/post url {:body               message
                          :throw-exceptions   false
-                         :content-type       :x-www-form-urlencoded
+                         :content-type       :json
                          :character-encoding "UTF-8"
                          :accept             :json}))))
 
@@ -244,15 +242,10 @@
           (string? mq-endpoint)
           (string? raw-command)]
    :post [(string? %)]}
-  (let [[msg id] (try
-                   (let [cmd (-> raw-command
-                                 (parse-command)
-                                 (annotate-command))]
-                     [(json/generate-string cmd) (get-in cmd [:annotations :id])])
-                   (catch com.fasterxml.jackson.core.JsonParseException e
-                     [raw-command (kitchensink/uuid)]))]
-    (with-open [conn (mq/connect! mq-spec)]
-      (mq/connect-and-publish! conn mq-endpoint msg))
+  (let [id (kitchensink/uuid)]
+    (with-open [conn (mq/activemq-connection mq-spec)]
+      (mq/connect-and-publish! conn mq-endpoint raw-command {"received" (kitchensink/timestamp)
+                                                             "id" id}))
     id))
 
 (defn enqueue-command!
@@ -267,7 +260,7 @@
           (string? command)
           (number? version)]
    :post [(string? %)]}
-  (with-open [conn (mq/connect! mq-spec)]
+  (with-open [conn (mq/activemq-connection mq-spec)]
     (let [command-map (annotate-command (assemble-command command version payload))]
       (mq/publish-json! conn mq-endpoint command-map)
       (get-in command-map [:annotations :id]))))
@@ -308,7 +301,7 @@
 (defn replace-catalog*
   [{:keys [payload annotations version]} {:keys [db catalog-hash-debug-dir]}]
   (let [catalog (upon-error-throw-fatality (cat/parse-catalog payload version))
-        certname (:certname catalog)
+        certname (:name catalog)
         id (:id annotations)
         timestamp (:received annotations)]
     (jdbc/with-transacted-connection' db :repeatable-read
@@ -318,9 +311,14 @@
         (scf-storage/replace-catalog! catalog timestamp catalog-hash-debug-dir)))
     (log/info (format "[%s] [%s] %s" id (command-names :replace-catalog) certname))))
 
+(defn warn-deprecated
+  "Logs a deprecation warning message for the given `command` and `version`"
+  [version command]
+  (log/warn (format "command '%s' version %s is deprecated, use the latest version" command version)))
+
 (defmethod process-command! [(command-names :replace-catalog) 1]
   [{:keys [version payload] :as command} options]
-  {:pre [(= version 1)]}
+  (warn-deprecated version "replace catalog")
   (when-not (string? payload)
     (throw (IllegalArgumentException.
              (format "Payload for a '%s' v1 command must be a JSON string."
@@ -328,23 +326,39 @@
   (replace-catalog* command options))
 
 (defmethod process-command! [(command-names :replace-catalog) 2]
-  [{:keys [version] :as  command} options]
-  {:pre [(= version 2)]}
+  [{:keys [version] :as command} options]
+  (warn-deprecated version "replace catalog")
   (replace-catalog* command options))
 
 (defmethod process-command! [(command-names :replace-catalog) 3]
-  [{:keys [version] :as  command} options]
-  {:pre [(= version 3)]}
+  [{:keys [version] :as command} options]
+  (warn-deprecated version "replace catalog")
+  (replace-catalog* command options))
+
+(defmethod process-command! [(command-names :replace-catalog) 4]
+  [{:keys [version] :as command} options]
   (replace-catalog* command options))
 
 ;; Fact replacement
 
 (defmethod process-command! [(command-names :replace-facts) 1]
+  [{:keys [version] :as command} config]
+  (warn-deprecated version "replace facts")
+  (-> command
+      (assoc :version 2)
+      (update-in [:payload] #(upon-error-throw-fatality (walk/keywordize-keys (json/parse-string %))))
+      (assoc-in [:payload :environment] nil)
+      (process-command! config)))
+
+(defmethod process-command! [(command-names :replace-facts) 2]
   [{:keys [payload annotations]} {:keys [db]}]
-  (let [{:strs [name] :as facts} (upon-error-throw-fatality (json/parse-string payload))
-        id                       (:id annotations)
-        timestamp                (:received annotations)]
-    
+  (let [{:keys [name values] :as facts} payload
+        ;; TODO: probably need to investigate if we really need to
+        ;; re-stringify this first.
+        facts     (upon-error-throw-fatality (update-in facts [:values] utils/stringify-keys))
+        id        (:id annotations)
+        timestamp (:received annotations)]
+
     (jdbc/with-transacted-connection' db :repeatable-read
       (scf-storage/maybe-activate-node! name timestamp)
       (scf-storage/replace-facts! facts timestamp))
@@ -353,14 +367,20 @@
 ;; Node deactivation
 
 (defmethod process-command! [(command-names :deactivate-node) 1]
-  [{:keys [payload annotations]} {:keys [db]}]
-  (let [certname (upon-error-throw-fatality (json/parse-string payload))
-        id       (:id annotations)]
-    (jdbc/with-transacted-connection db
-      (when-not (scf-storage/certname-exists? certname)
-        (scf-storage/add-certname! certname))
-      (scf-storage/deactivate-node! certname))
-    (log/info (format "[%s] [%s] %s" id (command-names :deactivate-node) certname))))
+  [{:keys [version] :as command} config]
+  (warn-deprecated version "deactivate node")
+  (-> command
+      (assoc :version 2)
+      (update-in [:payload] #(upon-error-throw-fatality (json/parse-string %)))
+      (process-command! config)))
+
+(defmethod process-command! [(command-names :deactivate-node) 2]
+  [{certname :payload {:keys [id]} :annotations} {:keys [db]}]
+  (jdbc/with-transacted-connection db
+    (when-not (scf-storage/certname-exists? certname)
+      (scf-storage/add-certname! certname))
+    (scf-storage/deactivate-node! certname))
+  (log/info (format "[%s] [%s] %s" id (command-names :deactivate-node) certname)))
 
 ;; Report submission
 
@@ -368,7 +388,7 @@
   [version db {:keys [payload annotations]}]
   (let [id          (:id annotations)
         report      (upon-error-throw-fatality
-                      (report/validate! version payload))
+                     (report/validate! version payload))
         certname    (:certname report)
         timestamp   (:received annotations)]
     (jdbc/with-transacted-connection db
@@ -380,14 +400,18 @@
 
 (defmethod process-command! [(command-names :store-report) 1]
   [{:keys [version] :as command} {:keys [db]}]
-  {:pre [(= version 1)]}
+  (warn-deprecated version "store report")
   (store-report* 1 db command))
 
 (defmethod process-command! [(command-names :store-report) 2]
-  [{:keys [version] :as command} {:keys [db]}]
-  {:pre [(= version 2)]}
+  [{:keys [version] :as command} {:keys [db] :as config}]
+  (warn-deprecated version "store report")
   (store-report* 2 db command))
 
+(defmethod process-command! [(command-names :store-report) 3]
+  [{:keys [version] :as command} {:keys [db]}]
+  (store-report* 3 db command))
+
 ;; ## MQ I/O
 ;;
 ;; The data flow through the code is as follows:
@@ -583,7 +607,6 @@
                    (rand-int (Math/pow 2 n)))
         error-msg (format "[%s] [%s] Retrying after attempt %d, due to: %s"
                           id command attempt e)]
-
     (if (> n (/ maximum-allowable-retries 4))
       (log/error e error-msg)
       (log/debug e error-msg))
