@@ -9,13 +9,13 @@
             [puppetlabs.kitchensink.ssl :as ssl]
             [puppetlabs.kitchensink.core :as kitchensink]
             [com.puppetlabs.time :as pl-time]
-            [com.puppetlabs.utils.logging :refer [configure-logging!]]
             [clj-time.core :as time]
             [clojure.java.io :as io]
             [fs.core :as fs]
             [clojure.string :as str]
             [schema.core :as s]
-            [com.puppetlabs.puppetdb.schema :as pls]))
+            [com.puppetlabs.puppetdb.schema :as pls]
+            [com.puppetlabs.puppetdb.utils :as utils]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Schemas
@@ -42,7 +42,8 @@
    (s/optional-key :partition-conn-max) (pls/defaulted-maybe s/Int 25)
    (s/optional-key :partition-count) (pls/defaulted-maybe s/Int 1)
    (s/optional-key :stats) (pls/defaulted-maybe String "true")
-   (s/optional-key :log-statements) (pls/defaulted-maybe String "true")})
+   (s/optional-key :log-statements) (pls/defaulted-maybe String "true")
+   (s/optional-key :statements-cache-size) (pls/defaulted-maybe s/Int 1000)})
 
 (def write-database-config-in
   "Includes the common database config params, also the write-db specific ones"
@@ -67,6 +68,7 @@
    :partition-count s/Int
    :stats pls/SchemaBoolean
    :log-statements pls/SchemaBoolean
+   :statements-cache-size s/Int
    (s/optional-key :conn-lifetime) (s/maybe pls/Minutes)
    (s/optional-key :username) String
    (s/optional-key :password) String
@@ -108,90 +110,6 @@
    (s/optional-key :store-usage) s/Int
    (s/optional-key :temp-usage) s/Int})
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Jetty config
-
-(defn configure-web-server-ssl-from-pems
-  "Configures the web server's SSL settings based on Puppet PEM files, rather than
-  via a java keystore (jks) file.  The configuration map returned by this function
-  will have overwritten any existing keystore-related settings to use in-memory
-  KeyStore objects, which are constructed based on the values of
-  `:ssl-key`, `:ssl-cert`, and `:ssl-ca-cert` from
-  the input map.  The output map does not include the `:puppet-*` keys, as they
-  are not meaningful to the web server implementation."
-  [{:keys [ssl-key ssl-cert ssl-ca-cert] :as jetty}]
-  {:pre  [ssl-key
-          ssl-cert
-          ssl-ca-cert]
-   :post [(map? %)
-          (instance? KeyStore (:keystore %))
-          (string? (:key-password %))
-          (instance? KeyStore (:truststore %))
-          (kitchensink/missing? % :trust-password :ssl-key :ssl-cert :ssl-ca-cert)]}
-  (let [old-ssl-config-keys [:keystore :truststore :key-password :trust-password]
-        old-ssl-config      (select-keys jetty old-ssl-config-keys)]
-    (when (pos? (count old-ssl-config))
-      (log/warn (format "Found settings for both keystore-based and Puppet PEM-based SSL; using PEM-based settings, ignoring %s"
-                  (keys old-ssl-config)))))
-  (let [truststore  (-> (ssl/keystore)
-                        (ssl/assoc-certs-from-file! "PuppetDB CA" ssl-ca-cert))
-        keystore-pw (kitchensink/uuid)
-        keystore    (-> (ssl/keystore)
-                        (ssl/assoc-private-key-file! "PuppetDB Agent Private Key" ssl-key keystore-pw ssl-cert))]
-    (-> jetty
-        (dissoc :ssl-key :ssl-ca-cert :ssl-cert :trust-password)
-        (assoc :keystore keystore)
-        (assoc :key-password keystore-pw)
-        (assoc :truststore truststore))))
-
-(defn jetty7-minimum-threads
-  "Given a thread count, make sure it meets the minimum count for Jetty 7 to
-  operate. It will return a warning if it does not, and return the minimum
-  instead of the original value.
-
-  This is to work-around a bug/feature in Jetty 7 that blocks the web server
-  when max-threads is less than the number of cpus on a system.
-
-  See: http://projects.puppetlabs.com/issues/22168 for more details.
-
-  This bug is solved in Jetty 9, so this check can probably be removed if we
-  upgrade."
-  ([threads]
-  (jetty7-minimum-threads threads (inc (kitchensink/num-cpus))))
-
-  ([threads min-threads]
-  {:pre [(pos? threads)
-         (pos? min-threads)]
-   :post [(pos? %)]}
-  (if (< threads min-threads)
-    (do
-      (log/warn (format "max-threads = %s is less than the minium allowed on this system for Jetty 7 to operate. This will be automatically increased to the safe minimum: %s"
-                  threads min-threads))
-      min-threads)
-    threads)))
-
-(defn configure-web-server
-  "Update the supplied config map with information about the HTTP webserver to
-  start. This will specify client auth, and add a default host/port
-  http://puppetdb:8080 if none are supplied (and SSL is not specified)."
-  [{:keys [jetty] :as config}]
-  {:pre  [(map? config)]
-   :post [(map? %)
-          (kitchensink/missing? (:jetty %) :ssl-key :ssl-cert :ssl-ca-cert)]}
-  (let [initial-config  {:max-threads 50}
-        merged-jetty    (merge initial-config jetty)
-        pem-required-keys [:ssl-key :ssl-cert :ssl-ca-cert]
-        pem-config        (select-keys jetty pem-required-keys)]
-    (assoc config :jetty
-      (-> (condp = (count pem-config)
-            3 (configure-web-server-ssl-from-pems jetty)
-            0 jetty
-            (throw (IllegalArgumentException.
-                     (format "Found SSL config options: %s; If configuring SSL from Puppet PEM files, you must provide all of the following options: %s"
-                        (keys pem-config) pem-required-keys))))
-          (assoc :client-auth :need)
-          (assoc :max-threads (jetty7-minimum-threads (:max-threads merged-jetty)))))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Database config 
 
@@ -362,13 +280,6 @@
       (assoc-in config [:global :catalog-hash-debug-dir] debug-dir))
     config))
 
-(defn assoc-when
-  "Assoc to `m` only when `k` is not already present in `m`"
-  [m k v]
-  (if (get m k)
-    m
-    (assoc m k v)))
-
 (defn normalize-product-name
   "Checks that `product-name` is specified as a legal value, throwing an
   exception if not. Returns `product-name` if it's okay."
@@ -389,36 +300,19 @@
                (fn [global-config]
                  (-> global-config
                      (assoc :product-name product-name)
-                     (assoc-when :event-query-limit 20000)
-                     (assoc-when :update-server "http://updates.puppetlabs.com/check-for-updates"))))))
+                     (utils/assoc-when :event-query-limit 20000
+                                       :update-server "http://updates.puppetlabs.com/check-for-updates"))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
 
-(defn parse-config
-  "Parses the given config file/directory and configures its various
-  subcomponents.
-
-  Also accepts an optional map argument 'initial-config'; if
-  provided, any initial values in this map will be included
-  in the resulting config map."
-  ([path]
-    (parse-config path {}))
-  ([path initial-config]
-    {:pre [(string? path)
-           (map? initial-config)]}
-    (let [file (io/file path)]
-      (when-not (.canRead file)
-        (throw (IllegalArgumentException.
-                (format "Configuration path '%s' must exist and must be readable." path)))))
-
-    (->> (kitchensink/inis-to-map path)
-         (merge initial-config)
-         configure-globals
-         configure-logging!
-         validate-vardir
-         configure-web-server
-         convert-config
-         configure-catalog-debugging)))
-
 
+(defn process-config!
+  "Accepts a map containing all of the user-provided configuration values
+  and configures the various PuppetDB subsystems."
+  [config]
+  (-> config
+      configure-globals
+      validate-vardir
+      convert-config
+      configure-catalog-debugging))
