@@ -62,7 +62,7 @@
       (-> query
           (json/parse-string true)
           ((partial query/query->sql version query-options))
-          ((partial query/limited-query-resource-events limit paging-options))
+          ((partial query/limited-query-resource-events version limit paging-options))
           (query-result-response)))
     (catch com.fasterxml.jackson.core.JsonParseException e
       (pl-http/error-response e))
@@ -95,9 +95,9 @@
   (-> (routes version)
     middleware/verify-accepts-json
     (middleware/validate-query-params {:required ["query"]
-                            :optional (concat
-                                        ["distinct-resources"
-                                         "distinct-start-time"
-                                         "distinct-end-time"]
-                                        paging/query-params)})
+                                       :optional (concat
+                                                   ["distinct-resources"
+                                                    "distinct-start-time"
+                                                    "distinct-end-time"]
+                                                   paging/query-params)})
     middleware/wrap-with-paging-options))
