@@ -45,6 +45,16 @@
                    ["=" ["node" "active"] true]]
                   req))
 
+(defn restrict-query-to-environment
+  "Restrict the query parameter of the supplied request so that it
+   only returns results for the supplied environment"
+  [environment req]
+  {:pre  [(string? environment)]
+   :post [(are-queries-different? req %)]}
+  (restrict-query ["and"
+                   ["=" "environment" environment]]
+                  req))
+
 (defn restrict-fact-query-to-name
   "Restrict the query parameter of the supplied request so that it
   only returns facts with the given name"
