@@ -4,7 +4,9 @@
             [com.puppetlabs.http :as pl-http]
             [com.puppetlabs.puppetdb.query.resources :as r]
             [ring.util.response :as rr]
-            [com.puppetlabs.cheshire :as json])
+            [com.puppetlabs.cheshire :as json]
+            [com.puppetlabs.puppetdb.http :refer [remove-environment remove-all-environments]]
+            [com.puppetlabs.puppetdb.query :as query])
   (:use [net.cgrand.moustache :only [app]]
         [com.puppetlabs.middleware :only (verify-accepts-json validate-query-params wrap-with-paging-options)]
         [com.puppetlabs.jdbc :only (with-transacted-connection get-result-count)]
@@ -12,8 +14,9 @@
 
 (defn munge-result-rows
   "Munge the result rows so that they will be compatible with the v2 API specification"
-  [rows]
-  (map #(clojure.set/rename-keys % {:file :sourcefile :line :sourceline}) rows))
+  [version]
+  (fn [rows]
+    (map #(clojure.set/rename-keys % {:file :sourcefile :line :sourceline}) rows)))
 
 (defn produce-body
   "Given a query, and database connection, return a Ring response with the query results.
@@ -25,20 +28,13 @@
       (let [parsed-query (json/parse-string query true)
             {[sql & params] :results-query
              count-query    :count-query} (r/query->sql version parsed-query paging-options)
-
-             resp (pl-http/json-response*
-                   (pl-http/streamed-response buffer
-                     ; NOTE - we we don't have a transaction here,
-                     ; then the outer transaction can end up being
-                     ; closed *after* the stream has been opened but
-                     ; before it's been read, causing a connection
-                     ; error when the caller tries to read it.
-                     (with-transacted-connection db
-                       (r/with-queried-resources sql params
-                         (case version
-                           :v1 (throw (IllegalArgumentException. "No support for v1"))
-                           :v2 (comp #(pl-http/stream-json % buffer) munge-result-rows)
-                           #(pl-http/stream-json % buffer))))))]
+            row-munge (case version
+                        :v1 (throw (IllegalArgumentException. "api v1 is retired"))
+                        :v2 (comp r/deserialize-params (munge-result-rows :v2))
+                        r/deserialize-params)
+            resp (pl-http/stream-json-results
+                  (fn [f]
+                    ((query/streamed-query-result db version sql params) (comp f row-munge))))]
 
         (if count-query
           (add-headers resp {:count (get-result-count count-query)})
@@ -77,7 +73,7 @@
 (defn resources-app
   [version]
   (case version
-    :v1 (throw (IllegalArgumentException. "no support for v1"))
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
     :v2 (build-resources-app
           (validate-query-params (query-app version) {:optional ["query"]}))
     (build-resources-app
