@@ -12,6 +12,7 @@
             [com.puppetlabs.puppetdb.http.facts :as facts]
             [com.puppetlabs.puppetdb.http.resources :as resources]
             [com.puppetlabs.puppetdb.http.nodes :as nodes]
+            [com.puppetlabs.puppetdb.http.environments :as envs]
             [net.cgrand.moustache :as moustache]))
 
 (def version :v3)
@@ -19,7 +20,7 @@
 (def v3-app
   (moustache/app
    ["commands"]
-   {:any cmd/command}
+   {:any (cmd/command-app version)}
 
    ["facts" &]
    {:any (facts/facts-app version)}
@@ -30,6 +31,9 @@
    ["nodes" &]
    {:any (nodes/node-app version)}
 
+   ["environments" &]
+   {:any (envs/environments-app version)}
+
    ["resources" &]
    {:any (resources/resources-app version)}
 
@@ -63,7 +67,7 @@
    {:any (aec/aggregate-event-counts-app version)}
 
    ["reports" &]
-   {:any reports/reports-app}
+   {:any (reports/reports-app version)}
 
    ["server-time" &]
    {:any st/server-time-app}))
