@@ -19,13 +19,14 @@
 ;; The `where` key is then inserted into a template query to return
 ;; the final result as a string of SQL code.
 ;;
-;; The compiled query components can be combined by operators such as `AND` or
-;; `OR`, which return the same sort of structure. Operators which accept other
-;; terms as their arguments are responsible for compiling their arguments
-;; themselves. To facilitate this, those functions accept as their first
-;; argument a map from operator to compile function. This allows us to have a
-;; different set of operators for resources and facts, or v1 and v2 resource
-;; queries, while still sharing the implementation of the operators themselves.
+;; The compiled query components can be combined by operators such as
+;; `AND` or `OR`, which return the same sort of structure. Operators
+;; which accept other terms as their arguments are responsible for
+;; compiling their arguments themselves. To facilitate this, those
+;; functions accept as their first argument a map from operator to
+;; compile function. This allows us to have a different set of
+;; operators for resources and facts, or queries, while still sharing
+;; the implementation of the operators themselves.
 ;;
 ;; Other operators include the subquery operators, `in`, `extract`, and
 ;; `select-resources` or `select-facts`. The `select-foo` operators implement
@@ -61,7 +62,12 @@
 ;;
 (ns com.puppetlabs.puppetdb.query
   (:require [clojure.string :as string]
-            [clojure.set :as set])
+            [clojure.set :as set]
+            [com.puppetlabs.puppetdb.http :refer [v4?]]
+            [puppetlabs.kitchensink.core :as kitchensink]
+            [com.puppetlabs.jdbc :as jdbc]
+            [clj-time.coerce :refer [to-timestamp]]
+            [com.puppetlabs.puppetdb.http :refer [remove-all-environments]])
   (:use [puppetlabs.kitchensink.core :only [parse-number keyset valset order-by-expr?]]
         [com.puppetlabs.puppetdb.scf.storage-utils :only [db-serialize sql-as-numeric sql-array-query-string sql-regexp-match sql-regexp-array-match]]
         [com.puppetlabs.jdbc :only [valid-jdbc-query? limited-query-to-vec query-to-vec paged-sql count-sql get-result-count]]
@@ -180,10 +186,7 @@
   [version ops & terms]
   {:post [(string? (:where %))]}
   (case version
-    :v1 (do
-          (when (empty? terms)
-            (throw (IllegalArgumentException. (str "not requires at least one term"))))
-          (negate-term* ops (cons "or" terms)))
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
     (do
       (when-not (= (count terms) 1)
         (throw (IllegalArgumentException. (format "'not' takes exactly one argument, but %d were supplied" (count terms)))))
@@ -193,11 +196,13 @@
 ;;  corresponding table names where the fields reside
 (def fact-columns {"certname" "certname_facts"
                    "name"     "certname_facts"
-                   "value"    "certname_facts"})
+                   "value"    "certname_facts"
+                   "environment" "certname_facts"})
 
 ;; This map's keys are the queryable fields for resources, and the values are the
 ;;  corresponding table names where the fields reside
 (def resource-columns {"certname"   "catalogs"
+                       "environment" "catalog_resources"
                        "catalog"    "catalog_resources"
                        "resource"   "catalog_resources"
                        "type"       "catalog_resources"
@@ -214,8 +219,52 @@
 
 ;; This map's keys are the queryable fields for nodes, and the values are the
 ;;  corresponding table names where the fields reside
-(def node-columns {"name"         "certnames"
-                   "deactivated"  "certnames"})
+(defn node-columns
+  "Return the queryable set of fields and corresponding table names where they reside"
+  [version]
+  (case version
+    (:v1 :v2 :v3)
+    {"name"                     "certnames"
+     "deactivated"              "certnames"}
+
+    {"name"                     "certnames"
+     "deactivated"              "certnames"
+     "facts_environment"   "certnames"
+     "report_environment"  "certnames"
+     "catalog_environment" "certnames"
+     "facts_timestamp"          "certnames"
+     "report_timestamp"         "certnames"
+     "catalog_timestamp"        "certnames"}))
+
+(defn node-std-fields
+  "Return the set of standard parameters (these support all operators) for nodes"
+  [version]
+  (case version
+    (:v1 :v2 :v3) #{"name"}
+    #{"name" "facts-environment" "catalog-environment" "report-environment"}))
+
+(def environments-columns {"name" "environments"})
+
+(def event-columns
+  {"certname"               ["reports"]
+   "configuration_version"  ["reports"]
+   "start_time"             ["reports" "run_start_time"]
+   "end_time"               ["reports" "run_end_time"]
+   "receive_time"           ["reports" "report_receive_time"]
+   "report"                 ["resource_events"]
+   "status"                 ["resource_events"]
+   "timestamp"              ["resource_events"]
+   "resource_type"          ["resource_events"]
+   "resource_title"         ["resource_events"]
+   "property"               ["resource_events"]
+   "new_value"              ["resource_events"]
+   "old_value"              ["resource_events"]
+   "message"                ["resource_events"]
+   "file"                   ["resource_events"]
+   "line"                   ["resource_events"]
+   "containment_path"       ["resource_events"]
+   "containing_class"       ["resource_events"]
+   "name"                   ["environments" "environment"]})
 
 (defn column-map->sql
   "Helper function that converts one of our column maps to a SQL string suitable
@@ -234,7 +283,7 @@
 (defmethod queryable-fields :resource
   [_ query-api-version]
   (case query-api-version
-    :v1 (throw (IllegalArgumentException. "queryable fields for resources not supported for v1"))
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
     :v2 (-> (keyset resource-columns)
             (set/union (keyset v3-renamed-resource-columns))
             (set/difference (valset v3-renamed-resource-columns)))
@@ -245,8 +294,16 @@
   (keyset fact-columns))
 
 (defmethod queryable-fields :node
+  [_ version]
+  (keyset (node-columns version)))
+
+(defmethod queryable-fields :environments
   [_ _]
-  (keyset node-columns))
+  (keyset environments-columns))
+
+(defmethod queryable-fields :event
+  [_ _]
+  (keyset event-columns))
 
 (def subquery->type
   {"select-resources" :resource
@@ -292,11 +349,13 @@
   {:post [valid-jdbc-query? %]}
   (let [{:keys [where params]} (compile-term ops query)
         sql (format "SELECT %s
-                       FROM (SELECT c.hash as catalog, catalog_id, resource,
-                                   type, title, tags, exported, file, line
-                               FROM catalog_resources cr, catalogs c
-                               WHERE c.id = cr.catalog_id) AS catalog_resources
-                       JOIN catalogs ON catalog_resources.catalog_id = catalogs.id WHERE %s"
+                       FROM (SELECT c.hash as catalog, e.name as environment, catalog_id, resource,
+                                    type, title, tags, exported, file, line
+                             FROM catalog_resources cr, catalogs c LEFT OUTER JOIN environments e
+                                  on c.environment_id = e.id
+                             WHERE c.id = cr.catalog_id) AS catalog_resources
+                       JOIN catalogs ON catalog_resources.catalog_id = catalogs.id
+                     WHERE %s"
                     (column-map->sql resource-columns) where)]
     (apply vector sql params)))
 
@@ -306,17 +365,64 @@
   [ops query]
   {:post [valid-jdbc-query? %]}
   (let [{:keys [where params]} (compile-term ops query)
-        sql (format "SELECT %s FROM certname_facts WHERE %s" (column-map->sql fact-columns) where)]
+        sql (format "SELECT %s FROM (select cf.certname, cf.name, cf.value, env.name as environment
+                                     FROM certname_facts cf INNER JOIN certname_facts_metadata cfm on cf.certname = cfm.certname
+                                                            LEFT OUTER JOIN environments as env on cfm.environment_id = env.id)
+                                     as certname_facts
+                     WHERE %s" (column-map->sql fact-columns) where)]
     (apply vector sql params)))
 
 (defn node-query->sql
   "Compile a node query, returning a vector containing the SQL and parameters
   for the query. All node columns are selected, and no order is applied."
+  [version ops query]
+  {:post [valid-jdbc-query? %]}
+  (let [sql (case version
+              (:v1 :v2 :v3)
+              (format "SELECT %s FROM certnames"
+                      (column-map->sql (node-columns version)))
+
+              (format "SELECT %s FROM (SELECT
+                       certnames.name,
+                       certnames.deactivated,
+                       catalogs.timestamp AS catalog_timestamp,
+                       certname_facts_metadata.timestamp AS facts_timestamp,
+                       reports.end_time AS report_timestamp,
+                       catalog_environment.name AS catalog_environment,
+                       facts_environment.name AS facts_environment,
+                       reports_environment.name AS report_environment
+                       FROM certnames
+                         LEFT OUTER JOIN catalogs
+                           ON certnames.name = catalogs.certname
+                         LEFT OUTER JOIN certname_facts_metadata
+                           ON certnames.name = certname_facts_metadata.certname
+                         LEFT OUTER JOIN reports
+                           ON certnames.name = reports.certname
+                             AND reports.hash
+                               IN (SELECT report FROM latest_reports)
+                         LEFT OUTER JOIN environments AS catalog_environment
+                           ON catalog_environment.id = catalogs.environment_id
+                         LEFT OUTER JOIN environments AS facts_environment
+                           ON facts_environment.id = certname_facts_metadata.environment_id
+                         LEFT OUTER JOIN environments AS reports_environment
+                           ON reports_environment.id = reports.environment_id) as certnames"
+                      (column-map->sql (node-columns version))))]
+    (if query
+      (let [{:keys [where params]} (compile-term ops query)
+            sql (str sql (format " WHERE %s" where))]
+        (apply vector sql params))
+      (vector sql))))
+
+(defn environments-query->sql
+  "Compile a environments query, returning a vector containing the SQL and parameters
+  for the query. All node columns are selected, and no order is applied."
   [ops query]
   {:post [valid-jdbc-query? %]}
-  (let [{:keys [where params]} (compile-term ops query)
-        sql (format "SELECT %s FROM certnames WHERE %s" (column-map->sql node-columns) where)]
-    (apply vector sql params)))
+  (let [{:keys [where params]} (when query (compile-term ops query))
+        projection (format "SELECT %s FROM environments " (column-map->sql environments-columns))
+        where-clause (when where
+                       (format "WHERE %s"  where))]
+    (apply vector (str projection where-clause) params)))
 
 (defn compile-resource-equality
   "Compile an = operator for a resource query. `path` represents the field
@@ -329,51 +435,47 @@
   (when-not (= (count args) 2)
     (throw (IllegalArgumentException. (format "= requires exactly two arguments, but %d were supplied" (count args)))))
   (case version
-    :v1 (do
-          ;; We call it "certname" in v2, and ["node" "name"] in v1. If they specify
-          ;; ["node" "name"], rewrite it as "certname". But if they specify "certname",
-          ;; fail because this is v1.
-          (when (= path "certname")
-            (throw (IllegalArgumentException. "certname is not a queryable object for resources")))
-          (let [path (if (= path ["node" "name"]) "certname" path)]
-            (compile-resource-equality :v2 path value)))
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
     :v2 (do
           ;; If they passed in any of the new names for the renamed resource-columns, we fail
           ;; because this is v2.
           (when (contains? (valset v3-renamed-resource-columns) path)
             (throw (IllegalArgumentException. (format "%s is not a queryable object for resources" path))))
           (compile-resource-equality :v3 (get v3-renamed-resource-columns path path) value))
-    (do
-      (match [path]
-             ;; tag join. Tags are case-insensitive but always lowercase, so
-             ;; lowercase the query value.
-             ["tag"]
-             {:where  (sql-array-query-string "tags")
-              :params [(string/lower-case value)]}
+    (match [path]
+           ;; tag join. Tags are case-insensitive but always lowercase, so
+           ;; lowercase the query value.
+           ["tag"]
+           {:where  (sql-array-query-string "tags")
+            :params [(string/lower-case value)]}
 
-             ;; node join.
-             ["certname"]
-             {:where  "catalogs.certname = ?"
-              :params [value]}
+           ;; node join.
+           ["certname"]
+           {:where  "catalogs.certname = ?"
+            :params [value]}
 
-             ;; {in,}active nodes.
-             [["node" "active"]]
-             {
-               :where (format "catalogs.certname IN (SELECT name FROM certnames WHERE deactivated IS %s)" (if value "NULL" "NOT NULL"))}
+           ["environment" :guard (v4? version)]
+           {:where  "catalog_resources.environment = ?"
+            :params [value]}
 
-             ;; param joins.
-             [["parameter" (name :guard string?)]]
-             {:where  "catalog_resources.resource IN (SELECT rp.resource FROM resource_params rp WHERE rp.name = ? AND rp.value = ?)"
-              :params [name (db-serialize value)]}
+           ;; {in,}active nodes.
+           [["node" "active"]]
+           {
+            :where (format "catalogs.certname IN (SELECT name FROM certnames WHERE deactivated IS %s)" (if value "NULL" "NOT NULL"))}
 
-             ;; metadata match.
-             [(metadata :guard #{"catalog" "resource" "type" "title" "tags" "exported" "file" "line"})]
-               {:where  (format "catalog_resources.%s = ?" metadata)
-                :params [value]}
+           ;; param joins.
+           [["parameter" (name :guard string?)]]
+           {:where  "catalog_resources.resource IN (SELECT rp.resource FROM resource_params rp WHERE rp.name = ? AND rp.value = ?)"
+            :params [name (db-serialize value)]}
 
-             ;; ...else, failure
-             :else (throw (IllegalArgumentException.
-                           (str path " is not a queryable object for resources")))))))
+           ;; metadata match.
+           [(metadata :guard #{"catalog" "resource" "type" "title" "tags" "exported" "file" "line"})]
+           {:where  (format "catalog_resources.%s = ?" metadata)
+            :params [value]}
+
+           ;; ...else, failure
+           :else (throw (IllegalArgumentException.
+                         (format "'%s' is not a queryable object for resources in the version %s API" path (last (name version))))))))
 
 (defn compile-resource-regexp
   "Compile an '~' predicate for a resource query, which does regexp matching.
@@ -383,7 +485,7 @@
   {:post [(map? %)
           (:where %)]}
   (case version
-    :v1 (throw (IllegalArgumentException. (format "No support for regexp for v1 api")))
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
     :v2 (do
           (when (contains? (valset v3-renamed-resource-columns) path)
             (throw (IllegalArgumentException. (format "%s cannot be the target of a regexp match" path))))
@@ -398,6 +500,10 @@
            {:where  (sql-regexp-match "catalogs.certname")
             :params [value]}
 
+           ["environment" :guard (v4? version)]
+           {:where  (sql-regexp-match "catalog_resources.environment")
+            :params [value]}
+
            ;; metadata match.
            [(metadata :guard #{"type" "title" "exported" "file"})]
            {:where  (sql-regexp-match (format "catalog_resources.%s" metadata))
@@ -405,55 +511,64 @@
 
            ;; ...else, failure
            :else (throw (IllegalArgumentException.
-                          (str path " cannot be the target of a regexp match"))))))
+                         (format "'%s' cannot be the target of a regexp match for version %s of the resources API" path (last (name version))))))))
 
 (defn compile-fact-equality
   "Compile an = predicate for a fact query. `path` represents the field to
   query against, and `value` is the value."
-  [path value]
-  {:post [(map? %)
-          (:where %)]}
-  (match [path]
-         ["name"]
-         {:where "certname_facts.name = ?"
-          :params [value]}
+  [version]
+  (fn [path value]
+    {:post [(map? %)
+            (:where %)]}
+    (match [path]
+           ["name"]
+           {:where "certname_facts.name = ?"
+            :params [value]}
 
-         ["value"]
-         {:where "certname_facts.value = ?"
-          :params [(str value)]}
+           ["value"]
+           {:where "certname_facts.value = ?"
+            :params [(str value)]}
 
-         ["certname"]
-         {:where "certname_facts.certname = ?"
-          :params [value]}
+           ["certname"]
+           {:where "certname_facts.certname = ?"
+            :params [value]}
 
-         [["node" "active"]]
-         {
-          :where (format "certname_facts.certname IN (SELECT name FROM certnames WHERE deactivated IS %s)" (if value "NULL" "NOT NULL"))}
-         :else
-         (throw (IllegalArgumentException. (str path " is not a queryable object for facts")))))
+           ["environment" :guard (v4? version)]
+           {:where "certname_facts.environment = ?"
+            :params [value]}
+
+           [["node" "active"]]
+           {:where (format "certname_facts.certname IN (SELECT name FROM certnames WHERE deactivated IS %s)" (if value "NULL" "NOT NULL"))}
+
+           :else
+           (throw (IllegalArgumentException. (format "%s is not a queryable object for version %s of the facts query api" path (last (name version))))))))
 
 (defn compile-fact-regexp
   "Compile an '~' predicate for a fact query, which does regexp matching.  This
   is done by leveraging the correct database-specific regexp syntax to return
   only rows where the supplied `path` match the given `pattern`."
-  [path pattern]
-  {:pre [(string? path)
-         (string? pattern)]
-   :post [(map? %)
-          (string? (:where %))]}
-  (let [query (fn [col] {:where (sql-regexp-match col) :params [pattern]})]
-    (match [path]
-           ["certname"]
-           (query "certname_facts.certname")
+  [version]
+  (fn [path pattern]
+    {:pre [(string? path)
+           (string? pattern)]
+     :post [(map? %)
+            (string? (:where %))]}
+    (let [query (fn [col] {:where (sql-regexp-match col) :params [pattern]})]
+      (match [path]
+             ["certname"]
+             (query "certname_facts.certname")
 
-           ["name"]
-           (query "certname_facts.name")
+             ["environment" :guard (v4? version)]
+             (query "certname_facts.environment")
 
-           ["value"]
-           (query "certname_facts.value")
+             ["name"]
+             (query "certname_facts.name")
 
-           :else (throw (IllegalArgumentException.
-                          (str path " is not a valid operand for regexp comparison"))))))
+             ["value"]
+             (query "certname_facts.value")
+
+             :else (throw (IllegalArgumentException.
+                           (format "%s is not a valid version %s operand for regexp comparison" path (last (name version)))))))))
 
 (defn compile-fact-inequality
   "Compile a numeric inequality for a fact query (> < >= <=). The `value` for
@@ -482,41 +597,33 @@
   [version path value]
   {:post [(map? %)
           (string? (:where %))]}
-  (case version
-    :v1 (match [path]
-          [["fact" (name :guard string?)]]
-          {:where  "certnames.name IN (SELECT cf.certname FROM certname_facts cf WHERE cf.name = ? AND cf.value = ?)"
-           :params [name (str value)]}
-          [["node" "active"]]
-          {:where (format "certnames.deactivated IS %s" (if value "NULL" "NOT NULL"))}
+  (match [path]
+         [(field :guard (node-std-fields version))]
+         {:where (format "%s = ?" (jdbc/dashes->underscores field))
+          :params [value] }
 
-          :else (throw (IllegalArgumentException.
-                         (str path " is not a queryable object for nodes"))))
-    (match [path]
-           ["name"]
-           {:where "certnames.name = ?"
-            :params [value]}
-           [["fact" (name :guard string?)]]
-           {:where  "certnames.name IN (SELECT cf.certname FROM certname_facts cf WHERE cf.name = ? AND cf.value = ?)"
-            :params [name (str value)]}
-           [["node" "active"]]
-           {:where (format "certnames.deactivated IS %s" (if value "NULL" "NOT NULL"))}
+         [["fact" (name :guard string?)]]
+         {:where  "certnames.name IN (SELECT cf.certname FROM certname_facts cf WHERE cf.name = ? AND cf.value = ?)"
+          :params [name (str value)]}
 
-           :else (throw (IllegalArgumentException.
-                          (str path " is not a queryable object for nodes"))))))
+         [["node" "active"]]
+         {:where (format "certnames.deactivated IS %s" (if value "NULL" "NOT NULL"))}
+
+         :else (throw (IllegalArgumentException.
+                       (str path " is not a queryable object for nodes")))))
 
 (defn compile-node-regexp
   "Compile an '~' predicate for a fact query, which does regexp matching.  This
   is done by leveraging the correct database-specific regexp syntax to return
   only rows where the supplied `path` match the given `pattern`."
-  [path pattern]
+  [version path pattern]
   {:pre [(string? pattern)]
    :post [(map? %)
           (string? (:where %))]}
   (let [query (fn [col] {:where (sql-regexp-match col) :params [pattern]})]
     (match [path]
-           ["name"]
-           {:where (sql-regexp-match "certnames.name")
+           [(field :guard (node-std-fields version))]
+           {:where (sql-regexp-match (jdbc/dashes->underscores field))
             :params [pattern]}
 
            [["fact" (name :guard string?)]]
@@ -524,7 +631,7 @@
             :params [name pattern]}
 
            :else (throw (IllegalArgumentException.
-                          (str path " is not a valid operand for regexp comparison"))))))
+                         (str path " is not a valid operand for regexp comparison"))))))
 
 (defn compile-node-inequality
   [op path value]
@@ -541,6 +648,146 @@
     (throw (IllegalArgumentException.
             (format "Value %s must be a number for %s comparison." value op)))))
 
+(defn compile-environments-equality
+  "Compile an equality operator for environments. This can either be for the value of
+  a specific fact, or based on node activeness."
+  [version path value]
+  {:post [(map? %)
+          (string? (:where %))]}
+  (case version
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
+    (:v2 :v3) (throw (IllegalArgumentException. (format "Querying for environments not supported in version %s" (last (name version)))))
+    (match [path]
+           ["name"]
+           {:where "environments.name = ?"
+            :params [value]}
+
+           :else (throw (IllegalArgumentException.
+                          (str path " is not a queryable object for environments"))))))
+
+(defn compile-environments-regexp
+  "Compile an '~' predicate for an environments query, which does regexp matching.  This
+  is done by leveraging the correct database-specific regexp syntax to return
+  only rows where the supplied `path` match the given `pattern`."
+  [path pattern]
+  {:pre [(string? pattern)]
+   :post [(map? %)
+          (string? (:where %))]}
+  (let [query (fn [col] {:where (sql-regexp-match col) :params [pattern]})]
+    (match [path]
+           ["name"]
+           {:where (sql-regexp-match "environments.name")
+            :params [pattern]}
+
+           :else (throw (IllegalArgumentException.
+                          (str path " is not a valid operand for regexp comparison"))))))
+
+(defn compile-resource-event-inequality
+  "Compile a timestamp inequality for a resource event query (> < >= <=).
+  The `value` for comparison must be coercible to a timestamp via
+  `clj-time.coerce/to-timestamp` (e.g., an ISO-8601 compatible date-time string)."
+  [& [op path value :as args]]
+  {:post [(map? %)
+          (string? (:where %))]}
+  (when-not (= (count args) 3)
+    (throw (IllegalArgumentException. (format "%s requires exactly two arguments, but %d were supplied" op (dec (count
+args))))))
+
+  (let [timestamp-fields {"timestamp"           "resource_events.timestamp"
+                          "run-start-time"      "reports.start_time"
+                          "run-end-time"        "reports.end_time"
+                          "report-receive-time" "reports.receive_time"}]
+    (match [path]
+      [(field :guard (kitchensink/keyset timestamp-fields))]
+      (if-let [timestamp (to-timestamp value)]
+        {:where (format "%s %s ?" (timestamp-fields field) op)
+         :params [(to-timestamp value)]}
+        (throw (IllegalArgumentException. (format "'%s' is not a valid timestamp value" value))))
+
+      :else (throw (IllegalArgumentException.
+                     (str op " operator does not support object '" path "' for resource events"))))))
+
+(defn compile-resource-event-equality
+  "Compile an = predicate for resource event query. `path` represents the field to
+  query against, and `value` is the value."
+  [version]
+  (fn [& [path value :as args]]
+    {:post [(map? %)
+            (string? (:where %))]}
+    (when-not (= (count args) 2)
+      (throw (IllegalArgumentException. (format "= requires exactly two arguments, but %d were supplied" (count args)))))
+    (let [path (jdbc/dashes->underscores path)]
+      (match [path]
+             ["certname"]
+             {:where (format "reports.certname = ?")
+              :params [value]}
+
+             ["latest_report?"]
+             {:where (format "resource_events.report %s (SELECT latest_reports.report FROM latest_reports)"
+                             (if value "IN" "NOT IN"))}
+
+             ["environment" :guard (v4? version)]
+             {:where "environments.name = ?"
+              :params [value]}
+
+             [(field :guard #{"report" "resource_type" "resource_title" "status"})]
+             {:where (format "resource_events.%s = ?" field)
+              :params [value] }
+
+             ;; these fields allow NULL, which causes a change in semantics when
+             ;; wrapped in a NOT(...) clause, so we have to be very explicit
+             ;; about the NULL case.
+             [(field :guard #{"property" "message" "file" "line" "containing_class"})]
+             (if-not (nil? value)
+               {:where (format "resource_events.%s = ? AND resource_events.%s IS NOT NULL" field field)
+                :params [value] }
+               {:where (format "resource_events.%s IS NULL" field)
+                :params nil })
+
+             ;; these fields require special treatment for NULL (as described above),
+             ;; plus a serialization step since the values can be complex data types
+             [(field :guard #{"old_value" "new_value"})]
+             {:where (format "resource_events.%s = ? AND resource_events.%s IS NOT NULL" field field)
+              :params [(db-serialize value)] }
+
+             :else (throw (IllegalArgumentException.
+                           (format "'%s' is not a queryable object for version %s of the resource events API" path (last (name version)))))))))
+
+(defn compile-resource-event-regexp
+  "Compile an ~ predicate for resource event query. `path` represents the field
+   to query against, and `pattern` is the regular expression to match."
+  [version]
+  (fn [& [path pattern :as args]]
+    {:post [(map? %)
+            (string? (:where %))]}
+    (when-not (= (count args) 2)
+      (throw (IllegalArgumentException. (format "~ requires exactly two arguments, but %d were supplied" (count args)))))
+    (let [path (jdbc/dashes->underscores path)]
+      (match [path]
+             ["certname"]
+             {:where (sql-regexp-match "reports.certname")
+              :params [pattern]}
+
+             ["environment" :guard (v4? version)]
+             {:where (sql-regexp-match "environments.name")
+              :params [pattern]}
+
+             [(field :guard #{"report" "resource_type" "resource_title" "status"})]
+             {:where  (sql-regexp-match (format "resource_events.%s" field))
+              :params [pattern] }
+
+             ;; these fields allow NULL, which causes a change in semantics when
+             ;; wrapped in a NOT(...) clause, so we have to be very explicit
+             ;; about the NULL case.
+             [(field :guard #{"property" "message" "file" "line" "containing_class"})]
+             {:where (format "%s AND resource_events.%s IS NOT NULL"
+                             (sql-regexp-match (format "resource_events.%s" field))
+                             field)
+              :params [pattern] }
+
+             :else (throw (IllegalArgumentException.
+                           (format "'%s' is not a queryable object for version %s of the resource events API" path (last (name version)))))))))
+
 (declare fact-operators)
 
 (defn resource-operators
@@ -548,21 +795,7 @@
   if the operator isn't known."
   [version]
   (case version
-    :v1 (fn [op]
-          (let [unsupported (fn [& args]
-                              (throw (IllegalArgumentException. (format "Operator '%s' is not available in v1 resource queries" op))))]
-            (condp = (string/lower-case op)
-              "=" (partial compile-resource-equality version)
-              "and" (partial compile-and (resource-operators version))
-              "or" (partial compile-or (resource-operators version))
-              "not" (partial compile-not version (resource-operators version))
-              ;; All the subquery operators are unsupported in v1, so we dispatch to a
-              ;; function that throws an exception
-              "extract" unsupported
-              "in" unsupported
-              "select-resources" unsupported
-              "select-facts" unsupported
-              nil)))
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
     (fn [op]
       (condp = (string/lower-case op)
         "=" (partial compile-resource-equality version)
@@ -580,45 +813,37 @@
   "Maps fact query operators to the functions implementing them. Returns nil
   if the operator isn't known."
   [version]
-  (fn [op]
-    (let [op (string/lower-case op)]
-      (cond
-        (#{">" "<" ">=" "<="} op)
-        (partial compile-fact-inequality op)
+  (case version
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
+    (fn [op]
+      (let [op (string/lower-case op)]
+        (cond
+          (#{">" "<" ">=" "<="} op)
+          (partial compile-fact-inequality op)
 
-        (= op "=") compile-fact-equality
-        (= op "~") compile-fact-regexp
-        ;; We pass this function along so the recursive calls know which set of
-        ;; operators/functions to use, depending on the API version.
-        (= op "and") (partial compile-and (fact-operators version))
-        (= op "or") (partial compile-or (fact-operators version))
-        (= op "not") (partial compile-not version (fact-operators version))
-        (= op "extract") (partial compile-extract version (fact-operators version))
-        (= op "in") (partial compile-in :fact version (fact-operators version))
-        (= op "select-resources") (partial resource-query->sql (resource-operators version))
-        (= op "select-facts") (partial fact-query->sql (fact-operators version))))))
+          (= op "=") (compile-fact-equality version)
+          (= op "~") (compile-fact-regexp version)
+          ;; We pass this function along so the recursive calls know which set of
+          ;; operators/functions to use, depending on the API version.
+          (= op "and") (partial compile-and (fact-operators version))
+          (= op "or") (partial compile-or (fact-operators version))
+          (= op "not") (partial compile-not version (fact-operators version))
+          (= op "extract") (partial compile-extract version (fact-operators version))
+          (= op "in") (partial compile-in :fact version (fact-operators version))
+          (= op "select-resources") (partial resource-query->sql (resource-operators version))
+          (= op "select-facts") (partial fact-query->sql (fact-operators version)))))))
 
 (defn node-operators
   "Maps node query operators to the functions implementing them. Returns nil
   if the operator isn't known."
   [version]
   (case version
-    :v1 (fn [op]
-          (let [op (string/lower-case op)
-                unsupported (fn [& args]
-                              (throw (IllegalArgumentException. (format "Operator '%s' is not available in v1 node queries" op))))]
-            (cond
-              (= op "=") (partial compile-node-equality version)
-              (#{">" "<" ">=" "<="} op) (partial compile-node-inequality op)
-              (= op "and") (partial compile-and (node-operators version))
-              (= op "or") (partial compile-or (node-operators version))
-              (= op "not") (partial compile-not version (node-operators version))
-              (#{"~" "extract" "in" "select-resources" "select-facts"} op) unsupported)))
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
     (fn [op]
       (let [op (string/lower-case op)]
         (cond
           (= op "=") (partial compile-node-equality version)
-          (= op "~") compile-node-regexp
+          (= op "~") (partial compile-node-regexp version)
           (#{">" "<" ">=" "<="} op) (partial compile-node-inequality op)
           (= op "and") (partial compile-and (node-operators version))
           (= op "or") (partial compile-or (node-operators version))
@@ -627,3 +852,64 @@
           (= op "in") (partial compile-in :node version (node-operators version))
           (= op "select-resources") (partial resource-query->sql (resource-operators version))
           (= op "select-facts") (partial fact-query->sql (fact-operators version)))))))
+
+(defn environments-operators
+  "Maps environment query operators to the functions implementing them. Returns nil
+  if the operator isn't known."
+  [version]
+  (case version
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
+    (fn [op]
+      (let [op (string/lower-case op)]
+        (cond
+          (= op "=") (partial compile-environments-equality version)
+          (= op "~") compile-environments-regexp
+          (= op "and") (partial compile-and (environments-operators version))
+          (= op "or") (partial compile-or (environments-operators version))
+          (= op "not") (partial compile-not version (environments-operators version))
+          (= op "extract") (partial compile-extract version (environments-operators version))
+          (= op "in") (partial compile-in :environments version (environments-operators version))
+          (= op "select-resources") (partial resource-query->sql (resource-operators version))
+          (= op "select-facts") (partial fact-query->sql (fact-operators version))
+          (#{">" "<" ">=" "<="} op) (fn [& [_ value]]
+                                      (throw (IllegalArgumentException. (format "Value %s must be a number for %s comparison." value op)))))))))
+
+(defn resource-event-ops
+  "Maps resource event query operators to the functions implementing them. Returns nil
+  if the operator isn't known."
+  [version]
+  (case version
+    :v1 (throw (IllegalArgumentException. "api v1 is retired"))
+    :v2 (throw (IllegalArgumentException. (str "Resource events end-point not available for api version " version)))
+    :v3 (fn [op]
+          (let [op (string/lower-case op)]
+            (cond
+              (= op "=") (compile-resource-event-equality version)
+              (= op "and") (partial compile-and (resource-event-ops version))
+              (= op "or") (partial compile-or (resource-event-ops version))
+              (= op "not") (partial compile-not version (resource-event-ops version))
+              (#{">" "<" ">=" "<="} op) (partial compile-resource-event-inequality op)
+              (= op "~") (compile-resource-event-regexp version))))
+    (fn [op]
+      (let [op (string/lower-case op)]
+        (cond
+          (= op "=") (compile-resource-event-equality version)
+          (= op "and") (partial compile-and (resource-event-ops version))
+          (= op "or") (partial compile-or (resource-event-ops version))
+          (= op "not") (partial compile-not version (resource-event-ops version))
+          (#{">" "<" ">=" "<="} op) (partial compile-resource-event-inequality op)
+          (= op "~") (compile-resource-event-regexp version)
+          (= op "extract") (partial compile-extract version (resource-event-ops version))
+          (= op "in") (partial compile-in :event version (resource-event-ops version))
+          (= op "select-resources") (partial resource-query->sql (resource-operators version))
+          (= op "select-facts") (partial fact-query->sql (fact-operators version)))))))
+
+(defn streamed-query-result
+  "Uses a cursored resultset (for streaming), removing environments when not
+   in version ;v4. Returns a function that accepts a single function. That function
+   with get the results of the query"
+  [db version sql params]
+  (fn [f]
+    (jdbc/with-transacted-connection db
+      (jdbc/with-query-results-cursor sql params rs
+        (f (remove-all-environments version rs))))))
