@@ -4,7 +4,7 @@
 
 (ns com.puppetlabs.puppetdb.query.population
   (:use [com.puppetlabs.jdbc :only (query-to-vec table-count with-transacted-connection)]
-        [com.puppetlabs.utils :only (quotient)]
+        [puppetlabs.kitchensink.core :only (quotient)]
         [metrics.gauges :only (gauge)]))
 
 (defn correlate-exported-resources
@@ -13,8 +13,8 @@
   []
   ;; TODO: This needs to only return results for active nodes
   (query-to-vec (str "SELECT DISTINCT exporters.type, exporters.title, "
-                     "(SELECT certname FROM certname_catalogs WHERE catalog=exporters.catalog) AS exporter, "
-                     "(SELECT certname FROM certname_catalogs WHERE catalog=collectors.catalog) AS collector "
+                     "(SELECT certname FROM catalogs WHERE id=exporters.catalog_id) AS exporter, "
+                     "(SELECT certname FROM catalogs WHERE id=collectors.catalog_id) AS collector "
                      "FROM catalog_resources exporters, catalog_resources collectors "
                      "WHERE exporters.resource=collectors.resource AND exporters.exported=true AND collectors.exported=false "
                      "ORDER BY exporters.type, exporters.title, exporter, collector ASC")))
@@ -24,8 +24,8 @@
   []
   {:post [(number? %)]}
   (-> (str "SELECT COUNT(*) AS c "
-           "FROM certname_catalogs cc, catalog_resources cr, certnames c "
-           "WHERE cc.catalog=cr.catalog AND c.name=cc.certname AND c.deactivated IS NULL")
+           "FROM catalogs clogs, catalog_resources cr, certnames c "
+           "WHERE clogs.id=cr.catalog_id AND c.name=clogs.certname AND c.deactivated IS NULL")
       (query-to-vec)
       (first)
       :c))
@@ -50,8 +50,8 @@
   []
   {:post [(number? %)]}
   (let [num-unique (-> (query-to-vec (str "SELECT COUNT(*) AS c FROM "
-                                          "(SELECT DISTINCT resource FROM catalog_resources cr, certname_catalogs cc, certnames c "
-                                          " WHERE cr.catalog=cc.catalog AND cc.certname=c.name AND c.deactivated IS NULL) r"))
+                                          "(SELECT DISTINCT resource FROM catalog_resources cr, catalogs clogs, certnames c "
+                                          " WHERE cr.catalog_id=clogs.id AND clogs.certname=c.name AND c.deactivated IS NULL) r"))
                        (first)
                        (:c))
         num-total  (num-resources)]
