@@ -15,8 +15,7 @@
                                     valid-jdbc-query?
                                     get-result-count
                                     paged-sql
-                                    count-sql
-                                    with-query-results-cursor]]
+                                    count-sql]]
         [com.puppetlabs.puppetdb.query :only [resource-query->sql
                                               resource-operators
                                               resource-columns]]
@@ -47,7 +46,7 @@
           sql (format (str "SELECT subquery1.certname, subquery1.resource, "
                                   "subquery1.type, subquery1.title, subquery1.tags, "
                                   "subquery1.exported, subquery1.file, "
-                                  "subquery1.line, rpc.parameters "
+                                  "subquery1.line, rpc.parameters, subquery1.environment "
                             "FROM (%s) subquery1 "
                             "LEFT OUTER JOIN resource_params_cache rpc "
                                 "ON rpc.resource = subquery1.resource")
@@ -69,12 +68,6 @@
   (let [parse-params #(if % (json/parse-string %) {})]
     (map #(update-in % [:parameters] parse-params) resources)))
 
-(defn with-queried-resources
-  [query params func]
-  (let [parse-params #(if % (json/parse-string %) {})]
-    (with-query-results-cursor query params rs
-      (func (deserialize-params rs)))))
-
 (defn limited-query-resources
   "Take a limit, and a map of SQL queries as produced by `query->sql`, return
   a map containing the results of the query, as well as optional metadata.
