@@ -61,7 +61,6 @@
                                                           sql-current-connection-database-name
                                                           sql-current-connection-database-version
                                                           postgres?
-                                                          pg-newer-than-8-1?
                                                           fix-identity-sequence]]))
 
 (defn- drop-constraints
@@ -220,10 +219,8 @@
     "CREATE INDEX idx_catalog_resources_type_title ON catalog_resources(type,title)")
 
   (when (postgres?)
-    (if (pg-newer-than-8-1?)
-      (sql/do-commands
-        "CREATE INDEX idx_catalog_resources_tags_gin ON catalog_resources USING gin(tags)")
-      (log/warn (format "Version %s of PostgreSQL is too old to support fast tag searches; skipping GIN index on tags. For reliability and performance reasons, consider upgrading to the latest stable version." (string/join "." (sql-current-connection-database-version)))))))
+    (sql/do-commands
+     "CREATE INDEX idx_catalog_resources_tags_gin ON catalog_resources USING gin(tags)")))
 
 (defn drop-old-tags-index
   "Drops the non-GIN tags index, which is never used (because nobody
@@ -419,8 +416,7 @@
 (defn drop-resource-tags-index
   "Remove the resource tags index, it can get very large and is not used"
   []
-  (when (pg-newer-than-8-1?)
-    (sql/do-commands "DROP INDEX IF EXISTS idx_catalog_resources_tags_gin")))
+  (sql/do-commands "DROP INDEX IF EXISTS idx_catalog_resources_tags_gin"))
 
 (defn use-bigint-instead-of-catalog-hash
   "This migration converts all catalog hash instances to use bigint sequences instead"
@@ -434,7 +430,7 @@
       catalog_version text NOT NULL,
       transaction_uuid character varying(255) DEFAULT NULL)"
 
-    ;; catalogs: Insert data from old table
+    ;; Catalogs: Insert data from old table
     "INSERT INTO catalogs_transform (hash, api_version, catalog_version, transaction_uuid)
       SELECT hash, api_version, catalog_version, transaction_uuid
         FROM catalogs"
@@ -634,7 +630,7 @@
 
             ;;Rename catalogs_transform to catalogs, replace constraints
             "ALTER TABLE catalogs_transform RENAME to catalogs"
-            
+
             (when (postgres?)
               "ALTER TABLE catalogs
                ADD CONSTRAINT catalogs_pkey PRIMARY KEY (id)")
@@ -643,7 +639,7 @@
              ADD CONSTRAINT catalog_resources_catalog_id_fkey FOREIGN KEY (catalog_id)
              REFERENCES catalogs (id)
              ON UPDATE NO ACTION ON DELETE CASCADE"
-            
+
             "ALTER TABLE catalogs
              ADD CONSTRAINT catalogs_certname_fkey FOREIGN KEY (certname)
              REFERENCES certnames (name)
@@ -663,6 +659,42 @@
 (defn reset-catalog-sequence-to-latest-id []
   (fix-identity-sequence "catalogs" "id"))
 
+(defn add-environments []
+  (sql/create-table :environments
+                    ["id" "bigserial NOT NULL PRIMARY KEY"]
+                    ["name" "TEXT NOT NULL" "UNIQUE"])
+
+  (sql/do-commands
+
+   "ALTER TABLE catalogs ADD environment_id integer"
+
+   (if (postgres?)
+     "ALTER TABLE catalogs ALTER COLUMN api_version DROP NOT NULL"
+     "ALTER TABLE catalogs ALTER COLUMN api_version SET NULL")
+
+   "ALTER TABLE certname_facts_metadata ADD environment_id integer"
+   "ALTER TABLE reports ADD environment_id integer"
+
+   "ALTER TABLE catalogs
+    ADD CONSTRAINT catalogs_env_fkey FOREIGN KEY (environment_id)
+    REFERENCES environments (id) ON UPDATE NO ACTION ON DELETE CASCADE"
+
+   "CREATE INDEX idx_catalogs_env ON catalogs(environment_id)"
+
+   "ALTER TABLE certname_facts_metadata
+    ADD CONSTRAINT facts_env_fkey FOREIGN KEY (environment_id)
+    REFERENCES environments (id) ON UPDATE NO ACTION ON DELETE CASCADE"
+
+   "CREATE INDEX idx_certname_facts_env ON certname_facts_metadata(environment_id)"
+
+   "ALTER TABLE reports
+    ADD CONSTRAINT reports_env_fkey FOREIGN KEY (environment_id)
+    REFERENCES environments (id) ON UPDATE NO ACTION ON DELETE CASCADE"
+
+   "CREATE INDEX idx_reports_env ON reports(environment_id)"
+   ))
+
+
 ;; The available migrations, as a map from migration version to migration function.
 (def migrations
   {1 initialize-store
@@ -685,7 +717,8 @@
    18 add-index-on-exported-column
    19 differential-edges
    20 differential-catalog-resources
-   21 reset-catalog-sequence-to-latest-id})
+   21 reset-catalog-sequence-to-latest-id
+   22 add-environments})
 
 (def desired-schema-version (apply max (keys migrations)))
 
