@@ -73,26 +73,31 @@
    :relationship edge-relationship-schema})
 
 (def catalog-schema
-  {:api-version s/Int
-   :version String
-   :certname String
-   :puppetdb-version s/Int
-   :resources resource-ref->resource-schema
-   :edges #{edge-schema}
-   (s/optional-key :transaction-uuid) (s/maybe String)})
+  "This is a bit of a hack to make a more restrictive schema in the storage layer.
+   Moving the more restrictive resource/edge schemas into puppetdb.catalogs is TODO. Upstream
+   code needs to assume a map of resources (not a vector) and tests need to be update to adhere
+   to the new format."
+  (assoc (cat/catalog-schema :all)
+    :resources resource-ref->resource-schema
+    :edges #{edge-schema}))
 
 (def fact-map-schema
   {String json-primitive-schema})
 
 (def facts-schema
-  {(s/required-key "name") String
-   (s/required-key "values") fact-map-schema
+  {(s/required-key :name) String
+   (s/required-key :values) fact-map-schema
    ;;These next two should not be necessary, it's due to a bug in the
    ;;terminus code.  Leaving this in until 2.0.  If the user hasn't
    ;;and the below two lines are in, it will fails (sees keys it
    ;;doesn't recognize).  Remove this at 2.0.
-   (s/optional-key "timestamp") s/Any
-   (s/optional-key "expiration") s/Any})
+   (s/optional-key :timestamp) s/Any
+   (s/optional-key :expiration) s/Any
+   (s/required-key :environment) (s/maybe s/Str)})
+
+(def environments-schema
+  {:id s/Int
+   :name s/Str})
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Schemas - Internal
@@ -180,6 +185,7 @@
    :gc                 (timer [ns-str "default" "gc-time"])
    :gc-catalogs        (timer [ns-str "default" "gc-catalogs-time"])
    :gc-params          (timer [ns-str "default" "gc-params-time"])
+   :gc-environments    (timer [ns-str "default" "gc-environments-time"])
 
    :updated-catalog    (counter [ns-str "default" "new-catalogs"])
    :duplicate-catalog  (counter [ns-str "default" "duplicate-catalogs"])
@@ -256,6 +262,30 @@
                      ["name=?" certname]
                      {:deactivated nil}))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Environments querying/updating
+
+
+(pls/defn-validated create-environment :- s/Int
+  "Inserts `env-name` into the environments table"
+  [env-name :- s/Str]
+  (:id (first (sql/insert-records :environments {:name env-name}))))
+
+(pls/defn-validated environment-id :- (s/maybe s/Int)
+  "Returns the id (primary key) from the environments table for the given `env-name`"
+  [env-name :- s/Str]
+  (sql/with-query-results rs ["select id from environments where name=?" env-name]
+    (:id (first rs))))
+
+(pls/defn-validated ensure-environment :- (s/maybe s/Int)
+  "Check if the given `env-name` exists, creates it if it does not. Always returns
+   the id of the `env-name` (whether created or existing)"
+  [env-name :- (s/maybe s/Str)]
+  (when env-name
+    (if-let [id (environment-id env-name)]
+      id
+      (create-environment env-name))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Catalog updates/changes
 
@@ -267,39 +297,38 @@
     ["SELECT id, hash FROM catalogs WHERE certname=?" certname]
     (first result-set)))
 
+(pls/defn-validated catalog-row-map
+  "Creates a row map for the catalogs table, optionally adding envrionment when it was found"
+  [hash
+   {:keys [api_version version transaction-uuid environment]} :- catalog-schema
+   timestamp :- pls/Timestamp]
+  {:hash hash
+   :api_version api_version
+   :catalog_version  version
+   :transaction_uuid transaction-uuid
+   :timestamp (to-timestamp timestamp)
+   :environment_id (ensure-environment environment)})
+
 (pls/defn-validated update-catalog-metadata!
   "Given some catalog metadata, update the db"
   [id :- Number
    hash :- String
-   {:keys [api-version version transaction-uuid]} :- catalog-schema
+   catalog :- catalog-schema
    timestamp :- pls/Timestamp]
   (sql/update-values :catalogs
                      ["id=?" id]
-                     {:hash hash
-                      :api_version      api-version
-                      :catalog_version  version
-                      :transaction_uuid transaction-uuid
-                      :timestamp (to-timestamp timestamp)}))
+                     (catalog-row-map hash catalog timestamp)))
 
 (pls/defn-validated add-catalog-metadata!
   "Given some catalog metadata, persist it in the db. Returns a map of the
   inserted data including any autogenerated columns."
   [hash :- String
-   {:keys [api-version version transaction-uuid certname]} :- catalog-schema
+   {:keys [name] :as catalog} :- catalog-schema
    timestamp :- pls/Timestamp]
   {:post [(map? %)]}
-  (let [return (first (sql/insert-records :catalogs
-                                          {:hash hash
-                                           :api_version api-version
-                                           :catalog_version version
-                                           :transaction_uuid transaction-uuid
-                                           :certname certname
-                                           :timestamp (to-timestamp timestamp)}))]
-
-    ;; PostgreSQL <= 8.1 does not support RETURNING so we fake it
-    (if (and (sutils/postgres?) (not (sutils/pg-newer-than-8-1?)))
-      (first (query-to-vec ["SELECT * FROM catalogs WHERE hash = ?" hash]))
-      return)))
+  (first (sql/insert-records :catalogs
+                             (assoc (catalog-row-map hash catalog timestamp)
+                               :certname name))))
 
 (pls/defn-validated resources-exist? :- #{String}
   "Given a collection of resource-hashes, return the subset that
@@ -576,12 +605,12 @@
 (pls/defn-validated update-catalog-associations!
   "Adds/updates/deletes the edges and resources for the given certname"
   [catalog-id :- Number
-   {:keys [resources edges certname]} :- catalog-schema
+   {:keys [resources edges name]} :- catalog-schema
    refs-to-hashes :- {resource-ref-schema String}]
   (time! (:add-resources metrics)
          (add-resources! catalog-id resources refs-to-hashes))
   (time! (:add-edges metrics)
-         (replace-edges! certname edges refs-to-hashes)))
+         (replace-edges! name edges refs-to-hashes)))
 
 (pls/defn-validated update-catalog-hash-miss
   "New catalogs for a given certname needs to have their metadata, resources and edges updated.  This
@@ -590,7 +619,7 @@
    hash :- String
    catalog :- catalog-schema
    refs-to-hashes :- {resource-ref-schema String}
-   catalog-hash-debug-dir :- (s/maybe String)
+   catalog-hash-debug-dir :- (s/maybe s/Str)
    timestamp :- pls/Timestamp]
 
   (inc! (:updated-catalog metrics))
@@ -619,8 +648,8 @@
    indicates where catalog debugging information should be stored."
   ([catalog :- catalog-schema]
      (add-catalog! catalog nil (now)))
-  ([{:keys [api-version resources edges certname] :as catalog} :- catalog-schema
-    catalog-hash-debug-dir :- (s/maybe String)
+  ([{:keys [api_version resources edges name] :as catalog} :- catalog-schema
+    catalog-hash-debug-dir :- (s/maybe s/Str)
     timestamp :- pls/Timestamp]
 
      (let [refs-to-hashes (time! (:resource-hashes metrics)
@@ -630,7 +659,7 @@
            hash           (time! (:catalog-hash metrics)
                                  (shash/catalog-similarity-hash catalog))
            {id :id
-            stored-hash :hash} (catalog-metadata certname)]
+            stored-hash :hash} (catalog-metadata name)]
 
        (sql/transaction
         (cond
@@ -665,12 +694,27 @@
   (time! (:gc-params metrics)
          (sql/delete-rows :resource_params_cache ["NOT EXISTS (SELECT * FROM catalog_resources cr WHERE cr.resource=resource_params_cache.resource)"])))
 
+
+(defn delete-unassociated-environments!
+  "Remove any environments that aren't associated with a catalog, report or factset"
+  []
+  (time! (:gc-environments metrics)
+         (sql/delete-rows :environments
+           ["ID NOT IN
+              (SELECT environment_id FROM catalogs
+               UNION ALL
+               SELECT environment_id FROM reports
+               UNION ALL
+               SELECT environment_id FROM certname_facts_metadata)"])))
+
 (defn garbage-collect!
   "Delete any lingering, unassociated data in the database"
   []
   (time! (:gc metrics)
          (sql/transaction
-          (delete-unassociated-params!))))
+          (delete-unassociated-params!))
+         (sql/transaction
+          (delete-unassociated-environments!))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Facts
@@ -689,10 +733,13 @@
   facts associated with the certname."
   [certname :- String
    facts :- fact-map-schema
-   timestamp :- pls/Timestamp]
+   timestamp :- pls/Timestamp
+   environment :- (s/maybe s/Str)]
   {:pre [(kitchensink/datetime? timestamp)]}
   (sql/insert-record :certname_facts_metadata
-                     {:certname certname :timestamp (to-timestamp timestamp)})
+                     {:certname certname
+                      :timestamp (to-timestamp timestamp)
+                      :environment_id (ensure-environment environment)})
   (insert-facts! certname facts))
 
 (pls/defn-validated delete-facts!
@@ -705,7 +752,7 @@
        (sql/delete-rows :certname_facts
                         (into [(str "certname=? and name " (jdbc/in-clause fact-names)) certname]  fact-names)))))
 
-(pls/defn-validated cert-fact-map
+(pls/defn-validated cert-fact-map :- fact-map-schema
   "Return all facts and their values for a given certname as a map"
   [certname :- String]
   (sql/with-query-results result-set
@@ -732,10 +779,12 @@
    to match the facts argument."
   [certname :- String
    facts :- fact-map-schema
-   timestamp :- pls/Timestamp]
+   timestamp :- pls/Timestamp
+   environment :- (s/maybe s/Str)]
   (let [old-facts (cert-fact-map certname)]
     (sql/update-values :certname_facts_metadata ["certname=?" certname]
-                       {:timestamp (to-timestamp timestamp)})
+                       {:timestamp (to-timestamp timestamp)
+                        :environment_id (ensure-environment environment)})
 
     (utils/diff-fn old-facts
                    facts
@@ -786,13 +835,22 @@
       #(not (or (empty? %) (kitchensink/string-contains? "[" %)))
       (reverse containment-path)))))
 
+(defn maybe-environment
+  "This fn is most to help in testing, instead of persisting a value of
+   nil, just omit it from the row map. For tests that are running older versions
+   of migrations, this function prevents a failure"
+  [row-map]
+  (if (nil? (:environment_id row-map))
+    (dissoc row-map :environment_id)
+    row-map))
+
 (defn add-report!*
   "Helper function for adding a report.  Accepts an extra parameter, `update-latest-report?`, which
   is used to determine whether or not the `update-latest-report!` function will be called as part of
   the transaction.  This should always be set to `true`, except during some very specific testing
   scenarios."
   [{:keys [puppet-version certname report-format configuration-version
-           start-time end-time resource-events transaction-uuid]
+           start-time end-time resource-events transaction-uuid environment]
     :as report}
    timestamp update-latest-report?]
   {:pre [(map? report)
@@ -811,15 +869,17 @@
     (time! (:store-report metrics)
            (sql/transaction
             (sql/insert-record :reports
-                               { :hash                   report-hash
-                                :puppet_version         puppet-version
-                                :certname               certname
-                                :report_format          report-format
-                                :configuration_version  configuration-version
-                                :start_time             (to-timestamp start-time)
-                                :end_time               (to-timestamp end-time)
-                                :receive_time           (to-timestamp timestamp)
-                                :transaction_uuid       transaction-uuid})
+                               (maybe-environment
+                                {:hash                   report-hash
+                                 :puppet_version         puppet-version
+                                 :certname               certname
+                                 :report_format          report-format
+                                 :configuration_version  configuration-version
+                                 :start_time             (to-timestamp start-time)
+                                 :end_time               (to-timestamp end-time)
+                                 :receive_time           (to-timestamp timestamp)
+                                 :transaction_uuid       transaction-uuid
+                                 :environment_id         (ensure-environment environment)}))
             (apply sql/insert-records :resource_events resource-event-rows)
             (if update-latest-report?
               (update-latest-report! certname))))))
@@ -832,23 +892,21 @@
   (sql/delete-rows :reports ["end_time < ?" (to-timestamp time)]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Database deprecation
+;;; Database support/deprecation
 
-(defmulti db-deprecated?
-  "Returns a vector with a boolean indicating if database type and version is
-  marked for deprecation. The second element in the vector is a string
-  explaining the deprecation."
-  (fn [dbtype version] dbtype))
+(defn db-deprecated?
+  "Returns a string with an deprecation message if the DB is deprecated,
+   nil otherwise."
+  []
+  (when (sutils/pg-8-4?)
+    "PostgreSQL DB 8.4 is deprecated and won't be supported in the future."))
 
-(defmethod db-deprecated? "PostgreSQL"
-  [_ version]
-  (if (pos? (compare [8 4] version))
-    [true "PostgreSQL DB 8.3 and older are deprecated and won't be supported in the future."]
-    [false nil]))
-
-(defmethod db-deprecated? :default
-  [_ _]
-  [false nil])
+(defn db-unsupported?
+  "Returns a string with an unsupported message if the DB is not supported,
+   nil otherwise."
+  []
+  (when (sutils/pg-older-than-8-4?)
+    "PostgreSQL DB versions 8.3 and older are no longer supported. Please upgrade Postgres and restart PuppetDB."))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
@@ -901,9 +959,9 @@
   ([catalog :- catalog-schema
     timestamp :- pls/Timestamp]
      (replace-catalog! catalog timestamp nil))
-  ([{:keys [certname] :as catalog} :- catalog-schema
+  ([{:keys [name] :as catalog} :- catalog-schema
     timestamp :- pls/Timestamp
-    catalog-hash-debug-dir :- (s/maybe String)]
+    catalog-hash-debug-dir :- (s/maybe s/Str)]
      (time! (:replace-catalog metrics)
             (sql/transaction
              (add-catalog! catalog catalog-hash-debug-dir timestamp)))))
@@ -914,13 +972,13 @@
    a repeatable read or serializable transaction enforces only one update to the facts of a certname
    can happen at a time.  The first to start the transaction wins.  Subsequent transactions will fail
    as the certname_facts_metadata will have changed while the transaction was in-flight."
-  [{:strs [name values]} :- facts-schema
+  [{:keys [name values environment]} :- facts-schema
    timestamp :- pls/Timestamp]
   (time! (:replace-facts metrics)
          (if-let [facts-meta-ts (certname-facts-metadata! name)]
            (when (.before facts-meta-ts (to-timestamp timestamp))
-             (update-facts! name values timestamp))
-           (add-facts! name values timestamp))))
+             (update-facts! name values timestamp environment))
+           (add-facts! name values timestamp environment))))
 
 (pls/defn-validated add-report!
   "Add a report and all of the associated events to the database."
@@ -928,12 +986,26 @@
    timestamp :- pls/Timestamp]
   (add-report!* report timestamp true))
 
-(defn warn-on-db-deprecation!
-  "Get metadata about the current connection and warn if the database we are
-  using is deprecated."
+(defn warn-on-db-deprecation
+  "Log a warning message if the database is deprecated"
   []
-  (let [version    (sutils/sql-current-connection-database-version)
-        dbtype     (sutils/sql-current-connection-database-name)
-        [deprecated? message] (db-deprecated? dbtype version)]
-    (when deprecated?
-      (log/warn message))))
+  (when-let [deprecated-message (db-deprecated?)]
+    (log/warn deprecated-message)))
+
+(defn fail-on-unsupported
+  "Log an error message to the log and console if the currently
+   configured database is unsupported, then call fail-fn  (probably to
+   exit)."
+  [fail-fn]
+  (let [msg (db-unsupported?)]
+    (when-let [attn-msg (and msg (utils/attention-msg msg))]
+      (utils/println-err attn-msg)
+      (log/error attn-msg)
+      (fail-fn))))
+
+(defn validate-database-version
+  "Checks to ensure that the database is supported, fails if supported, logs
+   if deprecated"
+  [action-for-unsupported-fn]
+  (fail-on-unsupported action-for-unsupported-fn)
+  (warn-on-db-deprecation))
